///////////////////////////////////////////////////////////////////////
//  Copyright (C) ILX Systems.  All Rights Reserved.
//
//      $Header: /u/4/m/me133/cvs/Common/ILXCommonPersistence.h,v 1.1 2004/09/07 06:10:49 me133 Exp $
//
//    File name:	IlxCommonPersistence.h
//   Created By:	Marc Eaddy
//   Created on:	10/4/99 5:45:38 PM
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: ILXCommonPersistence.h,v $
 * Revision 1.1  2004/09/07 06:10:49  me133
 * o Initial checkin to CVS
 * 
 * 
 * 13    4/30/01 3:53p Marce
 * o Added support for IgnorePersistedProperty.  Each control should store
 * the m_setDispidsToIgnore property, update it when
 * IgnorePersistedProperty is called, and pass it to
 * ILXCommonLoadFromStream() and ILXCommonSaveToStream().
 * 
 * 11    7/03/00 1:01p Marce
 * o Incremented version number since we changed the way we skip
 * properties when GetProperty() fails
 * o Modified Load() to skip VT_EMPTY properties.  VT_EMPTY is what we
 * save when GetProperty() fails.
 * o Now always writes a variant value during save even if we fail to
 * retrieve the property.
 * 
 * 10    6/26/00 6:07p Marce
 * o Incremented the version number
 * o RemoveSymbolFromStream only modifies streams with the new version
 * number so that it is compatible with old versions of controls.
 * o Added new argument to ILXCommonLoadFromStream so that we don't have
 * to use a global variable
 * 
 * 9     6/22/00 3:35p Marce
 * o Added RemoveSymbolFromStreamKludge()
 * 
 * 8     6/22/00 3:03p Marce
 * o Rolled-back to previous version by removing
 * ILXCommonLoadFromStream2().  No one should be calling this function
 * now.
 * 
 * 7     6/19/00 7:24p Marce
 * o Changed typedef name
 * 
 * 6     6/19/00 7:05p Marce
 * o Comments
 * 
 * 5     6/19/00 6:58p Marce
 * o Now buffers data directory, instance name, and mode
 * 
 * 4     2/11/00 12:51p Christ
 * 
 * 3     2/11/00 12:32p Chris
 * Update the version number for headline also expose version number
 * outside of this module.
 * 
 * 1     10/04/99 5:47p Marce
 */

#ifndef INCLUDED_ILX_COMMON_PERSISTENCE_H
#define INCLUDED_ILX_COMMON_PERSISTENCE_H

#include <atlbase.h>

#include <set>			// STL set<>

// Type defines
typedef std::set<long>	TypeSetDispids;

// Constants

const DWORD nILX_COMMON_PERSIST_BEGIN_VER	= 0x0701;
const DWORD nILX_COMMON_PERSIST_VER			= 0x0704;

// Forward declarations

struct IStream;
struct IUnknown;

// We have to use the ATL namespace.
namespace ATL
{
	struct ATL_PROPMAP_ENTRY;
}

HRESULT ILXCommonSaveToStream(	IStream* pStm, 
								ATL_PROPMAP_ENTRY* pMap,
								void* pThis,
								IUnknown* pUnk,
								const TypeSetDispids* psetDispidsToIgnore = 0);

HRESULT ILXCommonLoadFromStream(IStream* pStm, 
								ATL_PROPMAP_ENTRY* pMap, 
								void* pThis, 
								IUnknown* pUnk,
								DWORD* pdwPersistenceFormat = 0,
								const TypeSetDispids* psetDispidsToIgnore = 0);

HRESULT RemoveSymbolFromStreamKludge(IStream* pStm);

#endif // INCLUDED_ILX_COMMON_PERSISTENCE_H