///////////////////////////////////////////////////////////////////////
//  Copyright (C)  1988-2000 ILX Systems.  All Rights Reserved.
//
//      $Header: /u/4/m/me133/cvs/Common/ILXDragDropImpl.cpp,v 1.1 2004/09/07 06:10:50 me133 Exp $
//
//    File name:	ILXDragDropImpl.cpp
//   Created By:	Rafael Cruz
//   Created On:	7/24/00 4:41:27 PM
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: ILXDragDropImpl.cpp,v $
 * Revision 1.1  2004/09/07 06:10:50  me133
 * o Initial checkin to CVS
 * 
 * 
 * 3     7/23/01 10:53a Billyr
 * 
 * 2     7/03/01 12:36p Billyr
 * 
 * 1     6/29/01 4:39p Billyr
 * 
 * 3     8/04/00 3:07p Rafaelc
 * Modified Copyright notice to also include year
 * 
 * 2     8/04/00 2:54p Rafaelc
 * Modified Copyirght notice to also show year
 * 
 * 1     7/24/00 4:41p Rafaelc
 */

#include "StdAfx.h"
#include "ILXDragDropImpl.h"

//*************************************************************************************************************
//*********************************************IDropTargetImpl*************************************************
IDropSourceImpl::~IDropSourceImpl()
{

}

BOOL IDropSourceImpl::OnBeginDrag(HWND hWnd, POINT* pPtStart /* = NULL */)
{
	BOOL	bDragging = FALSE;
	POINT	ptStart;

	// set start point
	::GetCursorPos(&ptStart);
	::ScreenToClient(hWnd, &ptStart);

	// set initial key state
	m_grfInitKeyState = 0;
	if (::GetKeyState(VK_LBUTTON) < 0)
		m_grfInitKeyState |= MK_LBUTTON;
	if (::GetKeyState(VK_RBUTTON) < 0)
		m_grfInitKeyState |= MK_RBUTTON;

	DWORD dwLastTick = GetTickCount();
	::SetCapture(hWnd);
	ATLTRACE(__T("IDropSourceImpl::OnBeginDrag.::SetCapture(hWnd)...\n"));

	while (!bDragging)
	{
		// Some applications steal capture away at random times
		if (::GetCapture() != hWnd)
			break;

		// Peek for next input message
		MSG msg;
		if (PeekMessage(&msg, NULL, WM_MOUSEFIRST, WM_MOUSELAST, PM_REMOVE) ||
			PeekMessage(&msg, NULL, WM_KEYFIRST,   WM_KEYLAST,   PM_REMOVE))
		{
			// Check for button cancellation (any button down will cancel)
			if (msg.message == WM_LBUTTONUP   || msg.message == WM_RBUTTONUP ||
				msg.message == WM_LBUTTONDOWN || msg.message == WM_RBUTTONDOWN)
				break;

			// Check for keyboard cancellation
			if (msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE)
				break;

			// Check for drag start transition
			bDragging = (msg.pt.x < ptStart.x - DD_DEFDRAGMINDIST || msg.pt.x > ptStart.x + DD_DEFDRAGMINDIST ||
                         msg.pt.y < ptStart.y - DD_DEFDRAGMINDIST || msg.pt.y > ptStart.y + DD_DEFDRAGMINDIST);
		}

		// If the user sits here for 900 miliseconds start the drag.
		if (::GetTickCount() - dwLastTick > 900)//200 miliseconds
			bDragging = TRUE;
	}
	
	::ReleaseCapture();
	ATLTRACE(__T("IDropSourceImpl::OnBeginDrag.::ReleaseCapture().\n"));

	if (pPtStart)
		*pPtStart = ptStart;

	return bDragging;
}
HRESULT IDropSourceImpl::SetDataObject(IDataObject* pDataObject)
{
	ATLTRACE(__T("IDropSourceImpl::SetDataObject.\n"));

	if(!pDataObject)
		return E_POINTER;
	m_spDataObject = pDataObject;
	return S_OK;
}

STDMETHODIMP IDropSourceImpl::GiveFeedback(DWORD dwEffect)
{
	ATLTRACE(__T("IDropSourceImpl::GiveFeedback.\n"));

	return DRAGDROP_S_USEDEFAULTCURSORS;
}

STDMETHODIMP IDropSourceImpl::QueryContinueDrag(BOOL fEscapePressed, DWORD grfKeyState)
{
	ATLTRACE(__T("IDropSourceImpl::QueryContinueDrag.\n"));

	// check escape key 
	if (fEscapePressed ) 
	{
		m_bDragging = false; // avoid unecessary cursor setting
		return DRAGDROP_S_CANCEL;
	}

	// check left/left-button up to end drag/drop and do the drop
	if ((grfKeyState & MK_LBUTTON) == 0)
	{
		if(m_bDragging)
		{
			ATLTRACE(__T("IDropSourceImpl::QueryContinueDrag.DRAGDROP_S_DROP\n"));
			return DRAGDROP_S_DROP;
		}
		else
		{
			ATLTRACE(__T("IDropSourceImpl::QueryContinueDrag.DRAGDROP_S_CANCEL\n"));
			return DRAGDROP_S_CANCEL;
		}
	}

	// otherwise, keep polling...
	return S_OK;
}

//*************************************************************************************************************
//*********************************************IDropTargetImpl*************************************************
IDropTargetImpl::~IDropTargetImpl()
{

}

BOOL IDropTargetImpl::CanHandleDropFormat(IDataObject* pDataObject)
{
	ATLTRACE(__T("IDropTargetImpl::CanHandleDropFormat\n"));

	FORMATETC fe = {CF_UNICODETEXT, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL};
	if (FAILED(pDataObject->QueryGetData(&fe)))
	{
		fe.cfFormat = CF_TEXT;
		if (SUCCEEDED(pDataObject->QueryGetData(&fe)))
			return TRUE;
		else
			return FALSE;	
	}
	else
		return TRUE;
}

STDMETHODIMP IDropTargetImpl::DragEnter(IDataObject *pDataObject, DWORD grfKeyState, POINTL pt, DWORD *pdwEffect)
{
	ATLTRACE(__T("IDropTargetImpl::DragEnter\n"));

	HRESULT hr = S_OK;
	m_bInTarget = true;
	if (CanHandleDropFormat(pDataObject))
	{
		hr = OnDragEnter(pDataObject, grfKeyState,pt,pdwEffect);
	}
	else
		*pdwEffect = DROPEFFECT_NONE;    

	return hr;
}
	
STDMETHODIMP  IDropTargetImpl::DragOver(DWORD grfKeyState, POINTL pt, DWORD *pdwEffect)
{
	ATLTRACE(__T("IDropTargetImpl::DragOver\n"));

	m_bInTarget = true;//maybe our own...
	HRESULT hr;
	hr = OnDragOver(grfKeyState, pt,pdwEffect);
	return hr;
}	

STDMETHODIMP  IDropTargetImpl::DragLeave()
{
	ATLTRACE(__T("IDropTargetImpl::DragLeave\n"));

	m_bInTarget = false;
	OnDragLeave();
	return S_OK;
}
	
STDMETHODIMP IDropTargetImpl::Drop(IDataObject *pDataObject, DWORD grfKeyState, POINTL pt, DWORD  *pdwEffect)
{
	ATLTRACE(__T("IDropTargetImpl::Drop\n"));
	//note: 
	//Only the parent can compare pUnks to determine a self drop.
	m_bInTarget = false;
    OnDrop(pDataObject, grfKeyState, pt,pdwEffect);
	return S_OK;
		
}

//msft
