//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/MorphPlugin/MorphPlugin.cs 11    6/02/06 2:10a Eaddy $
//
// Description:
//
// Usage:
//
// Remarks:
//
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/MorphPlugin/MorphPlugin.cs $
 * 
 * 11    6/02/06 2:10a Eaddy
 * o Got rid of redundant verbosity flags
 * 
 * 10    5/24/06 10:18p Eaddy
 * o Verbosity of 0 now surpresses all messages
 * 
 * 9     5/24/06 9:48p Eaddy
 * o Now prints diagnostic messages in color
 * o Verbosity is now static
 * 
 * 8     5/03/06 12:22a Eaddy
 * o Trivial change
 * 
 * 7     5/02/06 4:51p Eaddy
 * o Made aspects/advice static so we can avoid loading it multiple times
 * in the dynamic weaving scenario
 * 
 * 6     4/20/06 4:44p Eaddy
 * o Can now specify 3 verbosity levels
 * 
 * 5     3/31/06 4:43p Asv2105
 * Fix for reuse of variable
 * 
 * 4     3/15/06 12:53p Eaddy
 * o Now MorphPlugin adds Summary Phase
 * 
 * 3     3/15/06 12:36p Eaddy
 * o Added verbosity flag
 * 
 * 2     2/24/06 8:19p Eaddy
 * o Now uses Phx.Aop.dll
 */

using System;
using System.IO;

namespace Phx.Morph
{
   class MorphPlugin : Phx.PlugIn
   {
      TypeMorphPhase morphPhase = null;
      string[] references = null;

      Phx.Ctrls.CompCtrl debugCtrl = null;
      Phx.Ctrls.StringCtrl refsCtrl = null;
      Phx.Ctrls.StringCtrl verboseCtrl = null;

      public override void RegisterObjects()
      {
         this.debugCtrl = Phx.Ctrls.CompCtrl.New("Morpher",
            "Morpher Debug Control", "MorphPlugin.cs");

         this.refsCtrl = Phx.Ctrls.StringCtrl.New(
            "r",
            "Reference metadata from the specified assembly files",
            "MorphPlugin.cs");

         this.verboseCtrl = Phx.Ctrls.StringCtrl.New(
            "v",
            "Specifies the verbosity of weaving messages: 0 (errors only-the default), 1 (errors and warnings) or 2 (all).",
            "MorphPlugin.cs");
      }

      public override void BuildPhases(Phx.Phases.PhaseConfig config)
      {
         // -r:<path>[,<path>]*
         string refs = this.refsCtrl.GetValue(null);
         if (refs != null && refs.Length > 0)
         {
            this.references = refs.Split(new char[] { ',' });
         }

         this.morphPhase = new TypeMorphPhase();
         this.morphPhase.Setup(config, this.references, this.debugCtrl);
          
         config.PhaseList.AppendPhase(SummaryPhase.New(config));

         string verbosityStr = this.verboseCtrl.GetValue(null);
         if (verbosityStr != null && verbosityStr.Length > 0)
         {
            int verbosity;
            if (!int.TryParse(verbosityStr, out verbosity) ||
               (verbosity < 0 || verbosity > 3))
            {
               Morpher.WriteError("Invalid verbosity: {0}. Expected 0, 1, 2, or 3.",
                  verbosityStr);
            }
            else
            {
               Helpers.Verbosity = verbosity;
            }
         }
      }

      public override System.String NameString
      {
         get
         {
            return "Morph";
         }
      }
   }
}
