//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/MorphPlugin/SummaryPhase.cs 2     5/24/06 9:48p Eaddy $
//
// Description:
//
//    Displays summary information and warnings related to weaving
//
// Remarks:
//
//    Just a trick to allow the TypeMorphPhase and WeavingPhase to
//    display warnings after all the function units have been processed.
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/MorphPlugin/SummaryPhase.cs $
 * 
 * 2     5/24/06 9:48p Eaddy
 * o Now prints diagnostic messages in color
 * o Verbosity is now static
 * 
 * 1     3/15/06 12:30p Eaddy
 * o Initial checkin
 */

using System;

using Dbg = System.Diagnostics.Debug;

namespace Phx.Morph
{
   class SummaryPhase : Phx.Phases.Phase
   {
      TypeMorphPhase typeMorphPhase = null;

      public static SummaryPhase New(Phx.Phases.PhaseConfig config)
      {
         Phx.Phases.Phase phaseFind = config.PhaseList.FindByName(TypeMorphPhase.Name);
         if (phaseFind == null || !(phaseFind is TypeMorphPhase))
         {
            Morpher.WriteError("'{0}' not found in phase list: '{1}'",
               TypeMorphPhase.Name, config.NameString);
            return null;
         }

         SummaryPhase phase = new SummaryPhase();
         phase.typeMorphPhase = phaseFind as TypeMorphPhase;
         Dbg.Assert(phase.typeMorphPhase != null);
         phase.Init(config, "Summary Phase");

         return phase;
      }

      protected override void Execute(Unit unit)
      {
         Dbg.Assert(typeMorphPhase != null);
         typeMorphPhase.ShowSummary();
      }
   }
}
