//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Aop/Advice.cs 8     5/05/06 6:18p Eaddy $
//
// Description:
//
// See:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Aop/Advice.cs $
 * 
 * 8     5/05/06 6:18p Eaddy
 * o Did some renaming to improve clarity
 * o Can now list current advice and enable/disable (although
 * enable/disable doesn't actually work yet)
 * 
 * 7     5/05/06 3:37p Eaddy
 * o Added attributes so the debugger skips over methods
 * 
 * 6     5/04/06 3:32p Eaddy
 * o Joinpoints now have source column so that we can weave
 * the same advice multiple times at the same joinpoint line 
 * 
 * 5     5/03/06 3:09p Rr2318
 * o Partially fixed bug where same advice needed to be applied to
 * multiple
 * locations in the same method.  Still have a problem if the advice needs
 * to be applied multiple times to the same line.
 * 
 * 4     5/03/06 12:29a Eaddy
 * o Now keeps track of where advice has been applied
 * 
 * 3     5/03/06 12:26a Eaddy
 * o Added Equals
 * 
 * 2     3/15/06 12:38p Eaddy
 * o Added ToString()
 * 
 * 1     3/01/06 2:17a Eaddy
 * o Now uses Phx.Aop library for pointcuts, advice and joinpoint matching
 */

using System.Collections.Generic;

using Dbg = System.Diagnostics.Debug;

using Phx.Aop.Attributes;
using Phx.Aop.Pointcuts;
using Phx.Aop.Joinpoints;

namespace Phx.Aop
{
   public class Advice
   {
      AdviceType adviceType;
      PointcutExpression pointcut;
      PointcutExpression pointcutExclusions = null;
      protected string adviceMethod = "";
      List<string> appliedJoinpoints = null;
      bool appliedToNewJoinpoints = false;
      bool enabled = true;

      public Advice(AdviceType adviceType, PointcutExpression pointcut)
      {
         this.adviceType = adviceType;
         this.pointcut = pointcut;
      }

      public Advice(AdviceAttribute attr)
      {
         adviceType = attr.AdviceType;
         pointcut = new NestedPointcutExpression();
         pointcut = pointcut.parse(attr.Pointcut);
      }

      public AdviceType AdviceType
      {
         get { return adviceType; }
      }

      public PointcutExpression Pointcut
      {
         [System.Diagnostics.DebuggerStepThrough]
         get { return pointcut; }
      }

      public PointcutExpression PointcutExclusions
      {
         [System.Diagnostics.DebuggerStepThrough]
         get { return pointcutExclusions; }
      }

      public string AdviceMethod
      {
         [System.Diagnostics.DebuggerStepThrough]
         get { return adviceMethod; }
      }

      public bool Enabled
      {
         [System.Diagnostics.DebuggerStepThrough]
         get { return enabled; }

         [System.Diagnostics.DebuggerStepThrough]
         set { enabled = value; }
      }

      public override string ToString()
      {
         return pointcut.ToString();
      }

      public bool AppliedToNewJoinpoints
      {
         get { return appliedToNewJoinpoints; }
         set { appliedToNewJoinpoints = value; }
      }

      public bool AlreadyApplied(IJoinpoint jp)
      {
         if (appliedJoinpoints == null)
            return false;
         else
            return appliedJoinpoints.Contains(getJoinpointKey(jp));
      }

      public void AddAppliedJoinpoint(IJoinpoint jp)
      {
         Dbg.Assert(!AlreadyApplied(jp));

         if (appliedJoinpoints == null)
            appliedJoinpoints = new List<string>();

         appliedJoinpoints.Add(getJoinpointKey(jp));
         appliedToNewJoinpoints = true;
      }

      static string getJoinpointKey(IJoinpoint jp)
      {
         return jp.Within + " [File:" + 
             jp.Within.SourceLocation.FileName + ", Line:" +
             jp.Within.SourceLocation.Line + ", Col:" +
             jp.Within.SourceLocation.Column + "]";
      }
      
      public List<string> AppliedJoinpoints
      {
         get { return appliedJoinpoints; }
      }

      public override bool Equals(object obj)
      {
         if (obj == null)
            return false;

         Advice advice = obj as Advice;
         return Equals(advice);
      }

      public bool Equals(Advice obj)
      {
         if (obj == null)
            return false;

         return this.adviceType == obj.adviceType &&
            this.ToString() == obj.ToString() &&
            this.adviceMethod == obj.adviceMethod;
      }

      public override int GetHashCode()
      {
         return adviceType.GetHashCode() ^ pointcut.GetHashCode() ^ adviceMethod.GetHashCode();
      }
   }
}
