//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Aop/Joinpoints/Joinpoint.cs 16    5/24/06 1:53a Eaddy $
//
// Description:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Aop/Joinpoints/Joinpoint.cs $
 * 
 * 16    5/24/06 1:53a Eaddy
 * o Breakpoint weaver now supports [Arg]
 * 
 * 15    5/24/06 1:34a Eaddy
 * o Added ArgNames to IJoinpointSignature so that we can
 * pass context args to the advice method
 * 
 * 14    5/04/06 4:59p Eaddy
 * o Changed from IEnumerable<object> to IEnumerator
 * 
 * 13    5/04/06 4:39p Eaddy
 * o Property names now make more sense
 * o Added properties for accessing attributes and args
 * as an object collection
 * 
 * 12    5/04/06 3:32p Eaddy
 * o Joinpoints now have source column so that we can weave
 * the same advice multiple times at the same joinpoint line 
 * 
 * 11    5/04/06 3:04p Eaddy
 * o Changed return type of Attributes property so that we aren't
 * locked in to how we implement it
 * 
 * 10    4/03/06 12:22a Eaddy
 * o Changed jp.WithinSignature to jp.Within and made it an
 * IJoinpointSignature.
 * This is needed because we need access to return type, arg types, etc.
 * just like we do for jp.Signature.
 * 
 * 9     4/02/06 5:23p Asv2105
 * Made so advice method is called with dynamic info pulled from
 * SimpleJoinpoint
 * 
 * Added first shot at filling SimpleJoinpoint with useful data
 * 
 * 8     4/02/06 10:49a Eaddy
 * o Adding support for args
 * 
 * 7     4/01/06 7:17p Eaddy
 * o Changed to 4 space tabs
 * 
 * 6     3/03/06 3:01p Eaddy
 * o Now supports file and line
 * 
 * 5     3/01/06 2:17a Eaddy
 * o Now uses Phx.Aop library for pointcuts, advice and joinpoint matching
 * 
 * 4     2/26/06 10:07p Eaddy
 * 
 * 3     2/26/06 9:13p Eaddy
 * o Added header comment
 */

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Phx.Aop.Joinpoints
{
    public enum JoinpointType { UNINITIALIZED, CALL, EXECUTION, GET, SET, STATEMENT };

    public interface ISourceLocation
    {
        StringBuilder FilePath { get; }
        string FileName { get; }
        uint Line { get; }
        uint Column { get; }
    }

    public interface IJoinpointSignature
    {
        IEnumerable<string> AttributeTypes { get; }
        IEnumerator Attributes { get; }
        IEnumerable<StringBuilder> ArgTypes { get; }
        IEnumerable<StringBuilder> ArgNames { get; }
        IEnumerator Args { get; }
        StringBuilder Access { get; }
        StringBuilder ReturnType { get; }
        StringBuilder DeclaringType { get; }
        StringBuilder MemberName { get; }
        ISourceLocation SourceLocation { get; }
    }

    public interface IJoinpoint
    {
        JoinpointType JoinpointType { get; }
        IJoinpointSignature Signature { get; }
        IJoinpointSignature Within { get; }
        StringBuilder Target { get; }
        StringBuilder This { get; }
        object getDataForAttribute(Type attributeType, object attributeVal);
    }
}
