//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Aop/Joinpoints/SimpleJoinpoint.cs 15    5/24/06 2:04p Eaddy $
//
// Description:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Aop/Joinpoints/SimpleJoinpoint.cs $
 * 
 * 15    5/24/06 2:04p Eaddy
 * o Reformated
 * o Added ToString()
 * 
 * 14    5/24/06 1:53a Eaddy
 * o Breakpoint weaver now supports [Arg]
 * 
 * 13    4/28/06 5:54p Asv2105
 * Fixed so data passed to contructor from other IJoinpoint is actually
 * copied.
 * 
 * 12    4/21/06 1:05p Asv2105
 * Added constructors to allow for cloning any IJoinpoint into a
 * SimpleJoinpoint.
 * 
 * 11    4/03/06 12:22a Eaddy
 * o Changed jp.WithinSignature to jp.Within and made it an
 * IJoinpointSignature.
 * This is needed because we need access to return type, arg types, etc.
 * just like we do for jp.Signature.
 * 
 * 10    4/02/06 11:21p Asv2105
 * Changed member vars to protected
 * 
 * 9     4/02/06 5:56p Asv2105
 * Updated PhxJoinpoint to call SimpleJoinpoint for getDataForAttribute
 * Updated PhxJoinpointSignature and SimpleJoinpointSignature to move
 * ToString() into SJS
 * 
 * 8     4/02/06 5:23p Asv2105
 * Made so advice method is called with dynamic info pulled from
 * SimpleJoinpoint
 * 
 * Added first shot at filling SimpleJoinpoint with useful data
 * 
 * 7     3/15/06 3:10p Eaddy
 * o Added accessors for SimpleXXX objects (C# doesn't support
 * multi-variant
 * return values?)
 * 
 * 6     3/03/06 3:01p Eaddy
 * o Now supports file and line
 * 
 * 5     3/01/06 2:17a Eaddy
 * o Now uses Phx.Aop library for pointcuts, advice and joinpoint matching
 * 
 * 4     2/26/06 10:07p Eaddy
 * 
 * 3     2/26/06 9:13p Eaddy
 * o Added header comment
 */

using System;
using System.Text;
using Phx.Aop.Attributes;

namespace Phx.Aop.Joinpoints
{
   public class SimpleJoinpoint : IJoinpoint
   {
      protected JoinpointType joinpointType;
      protected SimpleJoinpointSignature signature = null;
      protected SimpleJoinpointSignature within = null;
      protected StringBuilder target;
      protected StringBuilder _this;

      public SimpleJoinpoint()
      {
         signature = new SimpleJoinpointSignature();
         within = new SimpleJoinpointSignature();
      }

      public SimpleJoinpoint(IJoinpoint jp)
      {
         signature = new SimpleJoinpointSignature(jp.Signature);
         within = new SimpleJoinpointSignature(jp.Within);
         target = new StringBuilder(jp.Target.ToString());
         _this = new StringBuilder(jp.This.ToString());
         joinpointType = jp.JoinpointType;
      }

      public JoinpointType JoinpointType
      {
         get { return joinpointType; }
         set { joinpointType = value; }
      }

      public IJoinpointSignature Signature
      {
         get { return signature; }
      }

      public IJoinpointSignature Within
      {
         get { return within; }
      }

      public SimpleJoinpointSignature SimpleSignature
      {
         get { return signature; }
      }

      public SimpleJoinpointSignature SimpleWithin
      {
         get { return within; }
      }

      public StringBuilder Target
      {
         get { return target; }
         set { target = value; }
      }

      public StringBuilder This
      {
         get { return _this; }
         set { _this = value; }
      }

      public override string ToString()
      {
         StringBuilder sb = new StringBuilder();
         sb.Append(JoinpointType.ToString());
         sb.Append(' ');
         sb.Append(Signature.ToString());
         sb.Append(" in ");
         sb.Append(Within.ToString());
         sb.Append(' ');
         sb.Append(Within.SourceLocation.ToString());

         return sb.ToString();
      }

      public virtual object getDataForAttribute(Type attributeType, object attributeVal)
      {
         return SimpleJoinpoint.getDataForAttribute(attributeType, attributeVal, this);
      }

      public static object getDataForAttribute(Type attributeType, object attributeVal,
          IJoinpoint jp)
      {
         if (attributeType == typeof(SignatureAttribute))
         {
            return jp.Signature.ToString();
         }
         else if (attributeType == typeof(SignatureAttribute.DeclaringTypeNameAttribute))
         {
            return jp.Signature.DeclaringType.ToString();
         }
         else if (attributeType == typeof(SignatureAttribute.NameAttribute))
         {
            return jp.Signature.MemberName.ToString();
         }
         else if (attributeType == typeof(SourceLocationAttribute))
         {
            return jp.Within.SourceLocation.ToString();
         }
         else if (attributeType == typeof(SourceLocationAttribute.FilePathAttribute))
         {
            return jp.Within.SourceLocation.FilePath.ToString();
         }
         else if (attributeType == typeof(SourceLocationAttribute.FileNameAttribute))
         {
            return jp.Within.SourceLocation.FileName;
         }
         else if (attributeType == typeof(SourceLocationAttribute.LineAttribute))
         {
            return jp.Within.SourceLocation.Line;
         }
         else if (attributeType == typeof(SourceLocationAttribute.WithinNameAttribute))
         {
            return jp.Within.MemberName.ToString();
         }
         else if (attributeType == typeof(SourceLocationAttribute.WithinSignatureAttribute))
         {
            return jp.Within.ToString();
         }
         else if (attributeType == typeof(SourceLocationAttribute.WithinTypeNameAttribute))
         {
            return jp.Within.DeclaringType.ToString();
         }
         else
         {
            return null;
         }
      }
   }
}
