//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Aop/Pointcuts/Expressions/BinaryPointcutExpression.cs 5     5/11/06 12:20p Eaddy $
//
// Description:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Aop/Pointcuts/Expressions/BinaryPointcutExpression.cs $
 * 
 * 5     5/11/06 12:20p Eaddy
 * o Minor
 * 
 * 4     2/26/06 9:12p Eaddy
 * o Now supports joinpoint matching
 * o Changed tabs to 3 spaces to conform with MS standard
 * 
 * 3     2/26/06 7:48p Eaddy
 * 
 * 2     2/24/06 8:17p Eaddy
 * o Initial checkin
 */

using System;

using Dbg = System.Diagnostics.Debug;

using Phx.Aop.Joinpoints;

namespace Phx.Aop.Pointcuts
{
   enum BinaryOperator { INVALID, AND, OR };

   class BinaryPointcutExpression : PointcutExpression
   {
      public PointcutExpression left = null;
      public BinaryPointcutExpression right = null;
      public BinaryOperator op = BinaryOperator.INVALID;

      public BinaryPointcutExpression(PointcutExpression left, BinaryOperator op, BinaryPointcutExpression right)
      {
         this.left = left;
         this.op = op;
         this.right = right;
      }

      public BinaryPointcutExpression(PointcutExpression left)
      {
         this.left = left;
      }

      public BinaryPointcutExpression()
      { }

      public override PointcutExpression parse(string expr)
      {
         throw new Exception("The method or operation is not implemented.");
      }

      public override bool innerMatches(string exp)
      {
         if (left == null)
            return false;
         else if (right == null)
            return left.matches(exp);
         else if (op == BinaryOperator.OR)
            return left.matches(exp) || right.matches(exp);
         else
            return left.matches(exp) && right.matches(exp);
      }

      [System.Diagnostics.DebuggerStepThrough]
      public override bool innerMatches(IJoinpoint exp)
      {
         if (left == null)
            return false;
         else if (right == null)
            return left.matches(exp);
         else if (op == BinaryOperator.OR)
            return left.matches(exp) || right.matches(exp);
         else
            return left.matches(exp) && right.matches(exp);
      }

      public override bool innerMatches(PointcutExpression exp)
      {
         if (left == null)
         {
            BinaryPointcutExpression rhs = exp as BinaryPointcutExpression;
            if (rhs == null)
               return false;
            else
               return rhs.left == null;
         }
         else if (right == null)
            return left.matches(exp);
         else if (op == BinaryOperator.OR)
            return left.matches(exp) || right.matches(exp);
         else
            return left.matches(exp) && right.matches(exp);
      }

      public override string innerToString()
      {
         if (left == null)
            return "";
         else if (right == null)
            return left.ToString();
         else
            return string.Format("({0} {1} {2})",
                left,
                op == BinaryOperator.OR ? "||" : op == BinaryOperator.AND ? "&&" : "??",
                right);
      }

      public static BinaryOperator parseOperator(string expr)
      {
         switch (expr)
         {
            case "&&":
               return BinaryOperator.AND;
            case "||":
               return BinaryOperator.OR;
            default:
               throw (new System.Exception("Unexpected operator got past RegEx parser:" +
                   expr));
         }
      }
   }
}
