//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Aop/Pointcuts/PointcutFactory.cs 4     2/26/06 9:12p Eaddy $
//
// Description:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Aop/Pointcuts/PointcutFactory.cs $
 * 
 * 4     2/26/06 9:12p Eaddy
 * o Now supports joinpoint matching
 * o Changed tabs to 3 spaces to conform with MS standard
 * 
 * 3     2/26/06 7:48p Eaddy
 * 
 * 2     2/24/06 8:17p Eaddy
 * o Initial checkin
 */

using System;
using System.Collections.Generic;

using Dbg = System.Diagnostics.Debug;

namespace Phx.Aop.Pointcuts
{
   public interface IPointcutDesignator
   {
      string Name { get; }
      PointcutExpression create();
   }

   class PointcutFactory
   {
      static List<IPointcutDesignator> registry = new List<IPointcutDesignator>();

      static public void registerPointcutDesignator(IPointcutDesignator pcd)
      {
         registry.Add(pcd);
      }

      static public PointcutExpression create(string designator)
      {
         if (designator == null || designator.Length <= 0)
            return null;

         if (registry.Count == 0)
         {
            System.Console.WriteLine("There are no registered pointcut designators!");
            return null;
         }

         bool isPositive = true;

         if (designator[0] == '!')
         {
            isPositive = false;
            designator = designator.Remove(0, 1);
         }

         foreach (IPointcutDesignator pcd in registry)
         {
            if (pcd.Name == designator)
            {
               PointcutExpression exp = pcd.create();
               exp.trueValue = isPositive;
               return exp;
            }
         }

         System.Console.Write("Unknown pointcut designator: '" + designator + "'. Expected: ");
         bool first = true;
         foreach (IPointcutDesignator pcd in registry)
         {
            if (!first)
               System.Console.Write(", ");
            System.Console.Write(pcd.Name);
            first = false;
         }

         System.Console.WriteLine(".");

         return null;
      }

      static public PointcutExpression parse(string expr)
      {
         int len = expr.Length;

         if (len == 0)
            return null; // Invalid: Empty
         else if (len < 2)
            return null; // Invalid
         else if (expr[len - 1] != ')')
            return null; // Invalid: Expected last char to be end paren

         int paren_start = 0;
         while (paren_start < len && expr[paren_start++] != '(') ;

         if (paren_start == 0)
            return null; // Invalid

         string designator = expr.Substring(0, paren_start - 1);

         Dbg.Assert(designator.Length > 0);

         if (registry.Count == 0)
         {
            System.Console.WriteLine("There are no registered pointcut designators!");
            return null;
         }

         PointcutExpression pcdExpr = create(designator);
         if (pcdExpr == null)
            return null;

         pcdExpr = pcdExpr.parse(expr.Substring(paren_start, len - paren_start - 1));
         if (pcdExpr == null)
         {
            System.Console.WriteLine("Failed to parse {0} pointcut: {1}.",
                designator, expr.Substring(paren_start, len - paren_start - 1));
            return null;
         }

         return pcdExpr;
      }
   }
}
