//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Morph/Aop/PhxJoinpoint.cs 11    5/24/06 1:53a Eaddy $
//
// Description:
//
// See:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Morph/Aop/PhxJoinpoint.cs $
 * 
 * 11    5/24/06 1:53a Eaddy
 * o Breakpoint weaver now supports [Arg]
 * 
 * 10    5/11/06 1:56p Eaddy
 * o For Execution and Statement, the jp signature
 * is the same as the within signature
 * 
 * 9     4/03/06 1:30a Eaddy
 * o Fixed bug where within was returning the wrong signature
 * 
 * 8     4/03/06 12:23a Eaddy
 * o Changed jp.WithinSignature to jp.Within and made it an
 * IJoinpointSignature.
 * This is needed because we need access to return type, arg types, etc.
 * just like we do for jp.Signature.
 * 
 * 7     4/02/06 7:59p Eaddy
 * o Now handles "new" calls
 * 
 * 6     4/02/06 5:56p Asv2105
 * Updated PhxJoinpoint to call SimpleJoinpoint for getDataForAttribute
 * Updated PhxJoinpointSignature and SimpleJoinpointSignature to move
 * ToString() into SJS
 * 
 * 5     4/02/06 5:23p Asv2105
 * Made so advice method is called with dynamic info pulled from
 * SimpleJoinpoint
 * 
 * Added first shot at filling SimpleJoinpoint with useful data
 * 
 * 4     3/03/06 5:04p Eaddy
 * o Improved ToString
 * 
 * 3     3/03/06 4:19p Eaddy
 * o Removed unused references
 * 
 * 2     3/03/06 3:00p Eaddy
 * 
 * 1     3/01/06 2:17a Eaddy
 * o Now uses Phx.Aop library for pointcuts, advice and joinpoint matching
 */

using System;
using System.Text;

using Dbg = System.Diagnostics.Debug;

using Phx.Morph;

namespace Phx.Aop.Joinpoints
{
   public class PhxJoinpoint : IJoinpoint
   {
      Phx.IR.Instr instr = null;
      Phx.Opcode opcode = null;
      PhxJoinpointSignature phxSig = null;
      PhxJoinpointSignature within = null;
      JoinpointType joinpointType = JoinpointType.UNINITIALIZED;

      public Phx.IR.Instr Instr 
      {
         get { return instr; }
         set
         {
            this.instr = value;
            this.opcode = this.instr.Opcode;

            // Reset state variables
            joinpointType = JoinpointType.UNINITIALIZED;

            if (phxSig != null)
            {
               phxSig.Instr = instr;
            }

            if (within != null)
            {
               within.Instr = instr;
            }
         }
      }

      public JoinpointType JoinpointType
      {
         get
         {
            if (joinpointType != JoinpointType.UNINITIALIZED)
            {
               return joinpointType;
            }

            if (  opcode == Phx.Targets.Archs.MSIL.Opcode.call ||
                  opcode == Phx.Targets.Archs.MSIL.Opcode.callvirt ||
                  opcode == Phx.Targets.Archs.MSIL.Opcode.newobj)
            {
               joinpointType = JoinpointType.CALL;
            }
            else if (opcode == Phx.Common.Opcode.EnterFunc ||
                     opcode == Phx.Targets.Archs.MSIL.Opcode.ret)
            {
               joinpointType = JoinpointType.EXECUTION;
            }
            else if (opcode == Phx.Targets.Archs.MSIL.Opcode.ldfld ||
                     opcode == Phx.Targets.Archs.MSIL.Opcode.ldsfld)
            {
               joinpointType = JoinpointType.GET;
            }
            else if (opcode == Phx.Targets.Archs.MSIL.Opcode.stfld ||
                     opcode == Phx.Targets.Archs.MSIL.Opcode.stsfld)
            {
               joinpointType = JoinpointType.SET;
            }
            else
            {
               joinpointType = JoinpointType.STATEMENT;
            }

            return joinpointType;
         }
      }

      public IJoinpointSignature Signature
      {
         get 
         {
            if (JoinpointType == JoinpointType.EXECUTION ||
               JoinpointType == JoinpointType.STATEMENT)
            {
               return Within;
            }

            if (phxSig == null)
            {
               phxSig = new PhxJoinpointSignature(false);
               phxSig.Instr = instr;
            }

            return phxSig;
         }
      }

      public IJoinpointSignature Within
      {
         get
         {
            if (within == null)
            {
               within = new PhxJoinpointSignature(true);
               within.Instr = instr;
            }

            return within;
         }
      }

      public StringBuilder Target
      {
         get
         {
            if (joinpointType == JoinpointType.EXECUTION)
               //|| joinpointType == JoinpointType.CONSTRUCTOR
               //|| joinpointType == JoinpointType.INITIALIZATION
               //|| joinpointType == JoinpointType.HANDLER
               //|| joinpointType == JoinpointType.ADVICE)
            {
               return This;
            }
            else
            {
               return Signature.DeclaringType;
            }
         }
      }

      public StringBuilder This
      {
         get
         {
            return within.DeclaringType;
         }
      }

      public override string ToString()
      {
         StringBuilder sb = new StringBuilder();
         sb.Append(JoinpointType.ToString());
         sb.Append(' ');
         sb.Append(Signature.ToString());
         sb.Append(" in ");
         sb.Append(Within.ToString());
         sb.Append(' ');
         sb.Append(Within.SourceLocation.ToString());

         return sb.ToString();
      }

      public object getDataForAttribute(Type attributeType, object attributeVal) {
          return SimpleJoinpoint.getDataForAttribute(attributeType, attributeVal, this);
      }
   }
}
