//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Morph/Aop/PhxJoinpointSignature.cs 30    6/02/06 2:37a Eaddy $
//
// Description:
//
// See:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Morph/Aop/PhxJoinpointSignature.cs $
 * 
 * 30    6/02/06 2:37a Eaddy
 * o Shouldn't show warnings since we expect
 * them to happen
 * 
 * 29    5/24/06 9:56p Eaddy
 * o Now tries to resolve assemblies that can't be loaded
 * o Improved instantiation of custom attributes and finding
 * the correct ctor for a given blob
 * 
 * 28    5/24/06 1:34a Eaddy
 * o Added ArgNames to IJoinpointSignature so that we can
 * pass context args to the advice method
 * 
 * 27    5/11/06 11:02p Rr2318
 * Modified greatly to support attributes and statement attributes that
 * don't have constructor-field parity
 * 
 * 26    5/11/06 3:39p Eaddy
 * o Forgot to subtract one for the this ptr
 * 
 * 25    5/11/06 12:07p Eaddy
 * o Fixed bug where we were including the type
 * of the this ptr as the first arg type
 * 
 * 24    5/05/06 4:10p Eaddy
 * 
 * 23    5/05/06 12:40p Eaddy
 * o Now uses a type spec so that we load assemblies appropriately
 * 
 * 22    5/05/06 10:03a Eaddy
 * o Cleanup and comments
 * 
 * 21    5/05/06 9:39a Eaddy
 * 
 * 20    5/05/06 9:38a Eaddy
 * o Minor changes
 * 
 * 19    5/04/06 9:51p Eaddy
 * o Statement annotations now work!
 * 
 * 18    5/04/06 7:28p Eaddy
 * o Fixed bug where attribute wrappers weren't getting cleared
 * 
 * 17    5/04/06 4:59p Eaddy
 * o Changed from IEnumerable<object> to IEnumerator
 * 
 * 16    5/04/06 4:42p Eaddy
 * o Fixed compile error
 * 
 * 15    5/04/06 4:39p Eaddy
 * o Property names now make more sense
 * o Added properties for accessing attributes and args
 * as an object collection
 * 
 * 14    5/04/06 4:04p Eaddy
 * o Can't figure out how to prevent statement annotations
 * from being associated with instrs after the first one
 * o Fixed bug where attributes were always empty
 * 
 * 13    5/04/06 3:32p Eaddy
 * o Joinpoints now have source column so that we can weave
 * the same advice multiple times at the same joinpoint line 
 * 
 * 12    5/04/06 3:08p Eaddy
 * o Added handler to TypeLoadException.  This will happen if that
 * attribute contains
 * a System.Type, MethodInfo, or other metadata class that internally
 * refer to 
 * tokens.  Since the tokens won't be in the EmittedAssembly, type loading
 * will fail.
 * 
 * 11    5/03/06 8:15p Rr2318
 * Some safety checks added
 * 
 * 10    5/03/06 3:40p Rr2318
 * Cleaned up debug messages
 * 
 * 9     5/03/06 12:53p Rr2318
 * Added Support for Hydrolyzed Attributes
 * 
 * 8     4/03/06 1:30a Eaddy
 * o Fixed bug where within was returning the wrong signature
 * 
 * 7     4/03/06 12:23a Eaddy
 * o Changed jp.WithinSignature to jp.Within and made it an
 * IJoinpointSignature.
 * This is needed because we need access to return type, arg types, etc.
 * just like we do for jp.Signature.
 * 
 * 6     4/02/06 10:53p Eaddy
 * o Now displays new instead of .ctor
 * 
 * 5     4/02/06 7:58p Eaddy
 * o Reversed last change. Treating ".ctor" like "new" is now done by
 * PhxJoinpointSignature
 * 
 * 4     4/02/06 6:59p Eaddy
 * o Fixed bug where the signature type and method name were empty in
 * toString
 * 
 * 3     4/02/06 5:56p Asv2105
 * Updated PhxJoinpoint to call SimpleJoinpoint for getDataForAttribute
 * Updated PhxJoinpointSignature and SimpleJoinpointSignature to move
 * ToString() into SJS
 * 
 * 2     3/03/06 5:05p Eaddy
 * o Hardened
 * 
 * 1     3/01/06 2:17a Eaddy
 * o Now uses Phx.Aop library for pointcuts, advice and joinpoint matching
 */

using System.Text;
using System.Collections;
using System.Collections.Generic;

using Dbg = System.Diagnostics.Debug;

using Phx.Morph;
using Phx.Aop.Attributes;

namespace Phx.Aop.Joinpoints
{
   /// <summary>
   /// Attribute Wrapper added for passing Attribute and Constructor Info
   /// </summary>
   public class AttributeWrapper
   {
      System.Attribute attr = null;
      Phx.Syms.FuncSym ctor = null;
      public byte[] blob = null;
      public object[] args = null;
      public bool useArgs = false;
      bool isStatement = false;
      public int blobPos = 1; //blob always starts 01 00 and ends 00 00

      public System.Attribute Attr
      {
         get { return attr; }
         set { attr = value; }
      }

      public Phx.Syms.FuncSym Ctor
      {
         get { return ctor; }
         set { ctor = value; }
      }

      public byte[] Blob
      {
         get { return blob; }
         set
         { 
            useArgs = false;
            blob = value;
            args = null;
            blobPos = 1;
         }
      }

      public object[] Args
      {
         set
         {
            useArgs = true;
            blob = null;
            args = value;
            blobPos = 0;
         }
      }

      public override string ToString()
      {
         return attr == null ? "(null)" : attr.ToString();
      }
      public bool IsStatement
      {
         get { return isStatement; }
         set { isStatement = value; }
      }
   }

   public class PhxJoinpointSignature : IJoinpointSignature
   {
      Phx.IR.Instr instr = null;
      Phx.Opcode opcode = null;
      SimpleJoinpointSignature sig = new SimpleJoinpointSignature();
      PhxSourceLocation location = null;
      bool isWithin;
      AttributeHelper attributeHelper = new AttributeHelper();
      List<string> attributeTypes = null;
      List<AttributeWrapper> attributeWrappers = null;

      delegate void AttributeFoundCallback(Phx.Syms.AttrSym attrSym,
                                             System.Type attrType,
                                             StatementAttribute statementAttribute);

      public PhxJoinpointSignature(bool isWithin)
      {
          this.isWithin = isWithin;
      }

      public Phx.IR.Instr Instr
      {
         get { return instr; }
         set
         {
            this.instr = value;
            this.opcode = this.instr.Opcode;

            this.attributeTypes = null;
            this.attributeWrappers = null;

            // Reset state variables
            if (sig != null)
               sig.Reset();

            if (location != null)
               location.Instr = instr;
         }
      }

      Phx.Syms.FuncSym MyFuncSym
      {
          get
          {
              if (instr == null)
                  return null;
              else if (isWithin)
                  return instr.FuncUnit == null ? null : instr.FuncUnit.FuncSym;
              else 
                  return instr.FuncSym;
          }
      }

      public IEnumerable<string> AttributeTypes
      {
         get
         {
            if (attributeTypes == null)
            {
               findAttributes(AddAttributeTypeCallback);
            }

            return attributeTypes;
         }
      }

      public IEnumerator Attributes
      {
         get
         {
            if (attributeWrappers == null)
            {
               findAttributes(AddAttributeWrapperCallback);
            }

            return attributeWrappers.GetEnumerator();
         }
      }

      public AttributeWrapper[] RawAttributes
      {
         get { return attributeWrappers.ToArray(); }
      }

      void AddAttributeTypeCallback(   Phx.Syms.AttrSym attrSym,
                                       System.Type attrType,
                                       StatementAttribute statementAttribute)
      {
         string attributeTypeName;

         if (statementAttribute != null)
         {
            attributeTypeName = statementAttribute.AttributeType;
         }
         else
         {
            attributeTypeName = Helpers.SafeGetName(attrSym.AttrType);
         }

         if (attributeTypeName.EndsWith("Attribute"))
            attributeTypeName = attributeTypeName.Remove(attributeTypeName.Length - "Attribute".Length);

         if (attributeTypes == null)
            attributeTypes = new List<string>(1);

         attributeTypes.Add(attributeTypeName);
      }

      void AddAttributeWrapperCallback(   Phx.Syms.AttrSym attrSym,
                                          System.Type attrType,
                                          StatementAttribute statementAttribute)
      {
         if (attributeWrappers == null)
            attributeWrappers = new List<AttributeWrapper>();

         AttributeWrapper aw = new AttributeWrapper();

         if (statementAttribute != null)
         {
            try
            {
               aw.Attr = statementAttribute.Hydrate();
               if (aw.Attr == null)
               {
                  Helpers.WriteError("Failed to create statement attribute '{0}'.",
                      statementAttribute.AttributeType);
                  return;
               }
            }
            catch (System.FormatException ex)
            {
               Helpers.WriteError("Failed to create statement attribute. " + ex.Message);
               return;
            }
            
            System.Reflection.ConstructorInfo ctorInfo = statementAttribute.CtorInfo;
            Dbg.Assert(ctorInfo != null);

            aw.Args = statementAttribute.Args;
            aw.IsStatement = true; //  we can't actually get the correct constructor here; 
                                   //  this flag tells us to reinvestigate
            aw.Ctor = ReflectionHelpers.ConvertConstructorInfoToPhxFuncSym(instr.FuncUnit.SymTable, ctorInfo);
            if (aw.Ctor == null)
            {
               aw.Ctor = ReflectionHelpers.ConvertConstructorInfoToPhxFuncSym(
                  instr.FuncUnit.ParentUnit.SymTable, ctorInfo);
            }
            if (aw.Ctor == null)
            {
               Helpers.WriteError(
                  "Error: Failed to create statement attribute constructor '{0}'.",
                  Helpers.SafeGetName(attrSym.AttrType));
               return;
            }

         }
         else
         {
            aw.Attr = attributeHelper.InstantiateAttribute(attrSym, attrType) as System.Attribute;
            Dbg.Assert(aw.Attr != null);

            aw.Blob = attrSym.Blob;
            Dbg.Assert(aw.Blob != null);

            System.Reflection.ConstructorInfo ci =
               ReflectionHelpers.FindMatchingConstructorForBlob(attrType, attrSym.Blob, true);

            if (ci != null)
            {
               aw.Ctor = ReflectionHelpers.ConvertConstructorInfoToPhxFuncSym(attrSym.Table, ci);
               Dbg.Assert(aw.Ctor != null);
            }
         }

         attributeWrappers.Add(aw);
      }

      /// <summary>
      /// Finds the attributes associated with this join point
      /// </summary>
      /// <remarks>
      /// Attributes = signature attributes + within statement attributes
      /// 
      /// Signature attributes are the attributes associated with the join
      /// point signature.  If the signature is a method call, then they
      /// are the attributes associated with the method we are calling
      /// minus the statement attributes.  Within statement attributes are
      /// the statement attributes associated with the method we are calling
      /// from (the within method) but only if the statement attribute's
      /// line number matches that of the join point.
      /// 
      /// Keep in mind that AttributeTypes and Attributes never contain an
      /// actual StatementAttribute since they are always converted to the
      /// contained attribute.
      /// </remarks>
      /// <param name="attributeFoundCallback">
      /// This delegate will be called whenever we find an attribute.  In
      /// the case of StatementAttributes, we only call the function if the
      /// line number matches.
      /// </param>

      void findAttributes(AttributeFoundCallback attributeFoundCallback)
      {
         // ISSUE-TODO-t-marcea-2006/05/04-A StatementAttribute should only match
         // the first instruction corresponding to the line.

         // ISSUE-BUG-t-marcea-2006/05/05-For some reason the IL-line mapping in
         // the PDB is not updated when we output the morphed module.  This causes
         // statement annotations to match the wrong lines.

         //uint lineNumberOfPreviousLine = Helpers.SafeGetSourceLine(instr.Prev);

         bool firstInstrForThisLine = true; //(lineNumberOfPreviousLine != SourceLocation.Line);

         uint lineNumberToCheckForStatementAttribute;
         if (firstInstrForThisLine)
            lineNumberToCheckForStatementAttribute = SourceLocation.Line;
         else
            lineNumberToCheckForStatementAttribute = 0;

         // Gather signature attributes. If we are in an execution join point
         // the signature is the same as within so don't bother.

         // ISSUE-TODO-t-marcea-2006/05/06-We only gather attributes for
         // CALL join points, need to handle GET, SET, etc.

         // False means ignores statement attributes
         getAttributesForFuncSym(MyFuncSym, false, 0, attributeFoundCallback);

         // Gather within statement attributes

         getAttributesForFuncSym(instr.FuncUnit.FuncSym, true,
            lineNumberToCheckForStatementAttribute, attributeFoundCallback);

      }

      void getAttributesForFuncSym(    Phx.Syms.FuncSym funcSym, 
                                       bool statementAttributesOnly,
                                       uint lineNumberToCheckForStatementAttribute,
                                       AttributeFoundCallback attributeFoundCallback)
      {
         //otherwise initialize
         Phx.Syms.AttrSymListExtensionObject attrSymListExtObj =
            Phx.Syms.AttrSymListExtensionObject.GetExtensionObject(funcSym);

         if (attrSymListExtObj == null || attrSymListExtObj.AttrSymList == null)
         {
            // No attributes found
            return;
         }

         foreach (Phx.Syms.Sym attrExtSym in attrSymListExtObj.AttrSymList)
         {
            if (attrExtSym == null ||
               !attrExtSym.IsAttrSym)
            {
               // Ignore PermissionSyms
               continue;
            }

            Phx.Syms.AttrSym attrSym = attrExtSym.AsAttrSym;
            if (attrSym == null || attrSym.AttrType == null || attrSym.AttrType.TypeSym == null)
            {
               Dbg.Assert(attrSym == null || attrSym.Kind == Phx.Syms.AttrSymKind.Marshaling);
               continue;
            }

            // Convert from Phx.Types.Type to System.Type

            // ISSUE-TODO-t-marcea-2005/07/23-Phx#? Remove Reflection
            // for unmanaged implementation

            System.Type attrType =
               ReflectionHelpers.ConvertPhxTypeToClrType(attrSym.AttrType);

            if (attrType == null)
            {
               Helpers.WriteError("Unknown attribute type '{0}'", 
                   Helpers.SafeGetName(attrSym.AttrType));
               continue;
            }

            if (attrType == typeof(StatementAttribute))
            {
               // Ignore the statement attribute when checking the attributes of a function
               // we are calling (call joinpoints), OR
               // If the line number is invalid
               if (!statementAttributesOnly || lineNumberToCheckForStatementAttribute == 0)
               {
                  continue;
               }

               StatementAttribute statementAttribute = 
                  attributeHelper.InstantiateAttribute(attrSym, attrType) as StatementAttribute;
               Dbg.Assert(statementAttribute != null);

               if (statementAttribute == null ||
                  statementAttribute.Line != lineNumberToCheckForStatementAttribute)
               {
                  continue;
               }

               attributeFoundCallback(attrSym, attrType, statementAttribute);
            }
            else if (!statementAttributesOnly)
            {
               attributeFoundCallback(attrSym, attrType, null);
            }
         }
      }

      public StringBuilder Access
      {
         get
         {
            if (sig.access != null && sig.access.Length > 0)
               return sig.access;

            Phx.Syms.Access access;

            if (IsFieldInstr() && !isWithin)
            {
               if (instr.DstOpnd1 != null &&
                  instr.DstOpnd1.Field != null &&
                  instr.DstOpnd1.Field.FieldSym != null)
               {
                  access = instr.DstOpnd1.Field.FieldSym.Access;
               }
               else
               {
                  sig.SetAccess("(NA)");
                  return sig.access;
               }
            }
            else
            {
               if (MyFuncSym != null)
               {
                  access = MyFuncSym.Access;
               }
               else
               {
                  sig.SetAccess("(NA)");
                  return sig.access;
               }
            }

            switch (access)
            {
               case Phx.Syms.Access.Private: sig.SetAccess("private"); break;
               case Phx.Syms.Access.Public: sig.SetAccess("public"); break;
               case Phx.Syms.Access.Family: sig.SetAccess("protected"); break;
               case Phx.Syms.Access.Assembly: sig.SetAccess("internal"); break;

               // If a call is made to a method in another assembly, Phx doesn't
               // know the access because it doesn't load the assembly and defaults
               // the access to _Illegal.  We assume that the method is public,
               // otherwise how would you be able to call it from another assembly?
               case Phx.Syms.Access._Illegal: sig.SetAccess("public"); break;

               default:
               {
                  Helpers.WriteError("Unknown access type: " + access.ToString());
                  sig.SetAccess("(unknown)");
                  break;
               }
            }

            return sig.access;
         }
      }

      public StringBuilder ReturnType
      {
         get
         {
            if (sig.returnType != null && sig.returnType.Length > 0)
               return sig.returnType;

            Phx.Types.Type type;

            if (IsFieldInstr() && !isWithin)
            {
               if (instr.DstOpnd1 != null &&
                  instr.DstOpnd1.Field != null &&
                  instr.DstOpnd1.Field.FieldSym != null &&
                  instr.DstOpnd1.Field.FieldSym.Type != null)
               {
                  type = instr.DstOpnd1.Field.FieldSym.Type;
               }
               else
               {
                  sig.SetReturnType("(null)");
                  return sig.returnType;
               }
            }
            else
            {
               if (MyFuncSym != null && MyFuncSym.FuncType != null)
               {
                  type = MyFuncSym.FuncType.RetType;
               }
               else
               {
                  sig.SetReturnType("(null)");
                  return sig.returnType;
               }
            }

            Helpers.ConvertAggregateToPrimary(ref type);
            Helpers.SafeGetName2(type, ref sig.returnType);
            return sig.returnType;
         }
      }

      public StringBuilder DeclaringType
      {
         get
         {
            if (sig.declaringType != null && sig.declaringType.Length > 0)
               return sig.declaringType;

            if (IsFieldInstr() && !isWithin)
            {
               if (instr.DstOpnd1 != null &&
                  instr.DstOpnd1.Field != null &&
                  instr.DstOpnd1.Field.FieldSym != null &&
                  instr.DstOpnd1.Field.FieldSym.EnclosingAggrType != null)
               {
                  Helpers.SafeGetName2(instr.DstOpnd1.Field.FieldSym.EnclosingAggrType, ref sig.declaringType);
               }
               else
               {
                  sig.SetDeclaringType("(null)");
               }
            }
            else
            {
               if (MyFuncSym != null && MyFuncSym.EnclosingAggrType != null)
               {
                  Helpers.SafeGetName2(MyFuncSym.EnclosingAggrType, ref sig.declaringType);
               }
               else
               {
                  sig.SetDeclaringType("(null)");
               }
            }

            return sig.declaringType;
         }
      }

      public StringBuilder MemberName
      {
         get
         {
            if (sig.memberName != null && sig.memberName.Length > 0)
               return sig.memberName;

            if (IsFieldInstr() && !isWithin)
            {
               if (instr.DstOpnd1 != null &&
                  instr.DstOpnd1.Field != null &&
                  instr.DstOpnd1.Field.FieldSym != null)
               {
                  sig.SetMemberName(instr.DstOpnd1.Field.FieldSym.NameString);
               }
               else
               {
                  sig.SetMemberName("(null)");
               }
            }
            else
            {
               if (MyFuncSym != null)
               {
                  if (MyFuncSym.NameString == ".ctor" ||
                      MyFuncSym.NameString == ".cctor")
                  {
                     sig.SetMemberName("new");
                  }
                  else
                  {
                     sig.SetMemberName(MyFuncSym.NameString);
                  }
               }
               else
               {
                  sig.SetMemberName("(null)");
               }
            }

            return sig.memberName;
         }
      }

      public IEnumerable<StringBuilder> ArgTypes
      {
         get
         {
            if (sig.argTypes != null)
               return sig.argTypes;

            // Fields don't have arguments, only methods do
            Dbg.Assert(!IsFieldInstr());

            if (MyFuncSym == null || MyFuncSym.FuncType == null)
               return null;

            Phx.Types.FuncArg funcArg = MyFuncSym.FuncType.ArgFuncArgList;

            sig.argTypes = new StringBuilder[MyFuncSym.FuncType.CountUserDefinedArgs()];

            if (sig.argTypes.Length > 0)
            {
               int i = 0;
               while (funcArg != null)
               {
                  if (!funcArg.IsThisPtr)
                  {
                     Phx.Syms.TypeSym funcArgTypeSym = Helpers.GetUnderlyingTypeSym(funcArg.Type);
                     Dbg.Assert(funcArgTypeSym != null);

                     Dbg.Assert(i < sig.argTypes.Length);

                     sig.argTypes[i++] = new StringBuilder(funcArgTypeSym.NameString);
                  }

                  funcArg = funcArg.Next;
               }
            }

            return sig.argTypes;
         }
      }

      public IEnumerable<StringBuilder> ArgNames
      {
         get
         {
            if (sig.argNames != null)
               return sig.argNames;

            // Fields don't have arguments, only methods do
            Dbg.Assert(!IsFieldInstr());

            if (MyFuncSym == null || MyFuncSym.FuncArgList == null)
               return null;

            Phx.Collections.FuncArgList.Iterator funcSymIter =
             new Phx.Collections.FuncArgList.Iterator(MyFuncSym.FuncArgList);

            // Skip over the return value slot
            funcSymIter.MoveNext();

            int count = 0;
            while (funcSymIter.MoveNext())
               ++count;

            if (sig.argNames == null || sig.argNames.Length != count)
            {
               sig.argNames = new StringBuilder[count];
            }

            funcSymIter.Reset();
            
            // Skip over return value slot
            funcSymIter.MoveNext(); 

            int i = 0;
            while (funcSymIter.MoveNext())
            {
               SimpleJoinpointSignature.SetStringBuilder(ref sig.argNames[i++], 
                  funcSymIter.Curr.Name.NameString);
            }

            return sig.argNames;
         }
      }

      public IEnumerator Args
      {
         get { return null; } // Not implemented
      }

      public ISourceLocation SourceLocation
      {
         get
         { 
             if (location == null)
             {
                 location = new PhxSourceLocation();
                 location.Instr = instr;
             }

             return location; 
         }
      }

      public override string ToString()
      {
         return DeclaringType + "." + MemberName;
      }

      public bool IsFieldInstr()
      {
         return opcode == Phx.Targets.Archs.MSIL.Opcode.ldfld ||
                 opcode == Phx.Targets.Archs.MSIL.Opcode.ldsfld ||
                 opcode == Phx.Targets.Archs.MSIL.Opcode.stfld ||
                 opcode == Phx.Targets.Archs.MSIL.Opcode.stsfld;
      }
   }

}
