//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Morph/Attributes/Handlers/AddAttributeHandler.cs 3     3/03/06 3:39p Eaddy $
//
// Description:
//
//    Handles adding:
//
//       Fields
//       Methods
//       Properties
//       Attributes (NYI)
//       Events (NYI)
//       Delegates (NYI)
//       Classes (NYI)
//
// Remarks:
//
//    NYI = Not Yet Implemented
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Morph/Attributes/Handlers/AddAttributeHandler.cs $
 * 
 * 3     3/03/06 3:39p Eaddy
 * o Name change
 * 
 * 2     2/24/06 8:23p Eaddy
 * o Now contains Attribute instead of deriving from it
 * o Uses Phx.Aop.dll which has generic AOP attributes
 * o Added checkin comment
 */

using System;
using System.Diagnostics;

using Dbg = System.Diagnostics.Debug;

using Phx.Morph.Editors;

using Phx.Aop.Attributes;

namespace Phx.Morph.Attributes
{
   public class AddAttributeHandler : MorphTypeAttributeHandlerBase, IAttributeHandler
   {
      AddAttribute attr;

      public AddAttributeHandler()
      {
         attr = new AddAttribute();
      }

      public string AttributeName { get { return "Phx.Aop.Attributes.AddAttribute"; } }

      public IExtensionAttribute Create(Phx.Syms.AttrSym attrSym, Type attrType, AttributeHelper helper)
      {
         AddAttributeHandler newHandler = new AddAttributeHandler();
         newHandler.attr = helper.InstantiateAttribute(attrSym, attrType) as AddAttribute;
         return newHandler;
      }

      /// <summary>
      /// Does this extension override a virtual method
      /// </summary>
      /// <remarks>
      /// Only applies to attributes attached to methods
      /// </remarks>

      public bool IsOverride
      {
         get
         {
            // Removed assert so it doesn't interfere with debugging
            // which retrieves all the properties automatically

            //Dbg.Assert(AttachedSym != null && AttachedSym.IsFuncSym);
            return this.attr.IsOverride;
         }

         set { this.attr.IsOverride = value; }
      }

      /// <summary>
      /// Override for the field/method access modified
      /// </summary>

      public Phx.Syms.Access Access
      {
         get { return Phx.Morph.ReflectionHelpers.ConvertClrFieldAttrToPhxAccess(this.attr.Access); }
         set { this.attr.Access = Phx.Morph.ReflectionHelpers.ConvertPhxAccessToClrFieldAttr(value); }
      }

      /// <summary>
      /// Adds the thing (field/method) to the type
      /// </summary>
      /// <param name="moduleEditor">Module where the type to be morphed lives</param>
      /// <param name="typeEditor">The Type to be morphed</param>
      /// <returns>true on success</returns>

      public override bool
      Morph
      (
         ModuleEditor moduleEditor,
         TypeEditor typeEditor
      )
      {
         Dbg.Assert(AttachedType != null);

         if (AttachedType.IsFuncType)
         {
            Dbg.Assert(AttachedSym != null);
            Dbg.Assert(AttachedSym.IsFuncSym);

            return typeEditor.AddMethod(AttachedSym.AsFuncSym,
                                          this.IsOverride,
                                          this.Access) != null;
         }
         else if (this.AttachedSym != null &&
            this.AttachedSym.IsFieldSym)
         {
            return typeEditor.AddField(this.AttachedSym.AsFieldSym) != null;
         }
         else if (this.AttachedSym != null &&
            this.AttachedSym.IsPropertySym)
         {
            return typeEditor.AddProperty(this.AttachedSym.AsPropertySym,
                                          this.Access) != null;
         }
         else
         {
            Phx.Syms.Access accessToUse = this.Access;
            if (accessToUse == Phx.Syms.Access._Illegal)
            {
               accessToUse = Phx.Syms.Access.Public;
            }

            return typeEditor.AddField(this.AttachedType,
                                          this.Name,
                                          accessToUse) != null;
         }
      }
   }
}
