//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/Morpher/Phx.Morph/Attributes/Handlers/AdviceAttributeHandler.cs 12    5/24/06 9:53p Eaddy $
//
// Description:
//
//    Handles weaving advice into a method
//
// Remarks:
//
//    Instead of having the WeavingPhase call AdviceAttribute.Morph(),
//    we first gather all the attributes and pass them into
//    MethodEditor.Weave() which is more efficient.
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Morpher/Phx.Morph/Attributes/Handlers/AdviceAttributeHandler.cs $
 * 
 * 12    5/24/06 9:53p Eaddy
 * o Better diagnostics when the ctor param list for the inner
 * attribute doesn't match any ctor
 * 
 * 11    5/24/06 8:56a Eaddy
 * o Changed file names but forgot to change class
 * names
 * 
 * 10    5/24/06 1:35a Eaddy
 * o Switched names for MethodWeaver and MethodEditor
 * so now they make sense
 * o Fixed bug where AppendFuncArg didn't work when
 * weaving static methods
 * 
 * 9     3/15/06 2:31p Eaddy
 * o Trivial
 * 
 * 8     3/15/06 1:20p Eaddy
 * o Change message
 * 
 * 7     3/03/06 3:38p Eaddy
 * o Name change
 * 
 * 6     3/03/06 3:35p Eaddy
 * o Name change
 * 
 * 5     3/03/06 2:57p Asv2105
 * Refactored so all classes refer to base class of MethodWeaver rather
 * than MethodEditor, and create new instances through
 * MethodWeaverFactory.
 * 
 * 4     3/01/06 2:17a Eaddy
 * o Now uses Phx.Aop library for pointcuts, advice and joinpoint matching
 * 
 * 3     2/24/06 8:23p Eaddy
 * o Now contains Attribute instead of deriving from it
 * o Uses Phx.Aop.dll which has generic AOP attributes
 * o Added checkin comment
 */

using System;

using Dbg = System.Diagnostics.Debug;

using Phx.Morph.Aop;
using Phx.Morph.Editors;

using Phx.Aop;
using Phx.Aop.Attributes;

namespace Phx.Morph.Attributes
{
   /// <summary>
   /// Handles weaving advice into a method
   /// </summary>

   public class AdviceAttributeHandler : AttributeHandlerBase, IAttributeHandler
   {
      PhxAdvice advice;

      public AdviceAttributeHandler()
      { }

      public string AttributeName { get { return "Phx.Aop.Attributes.AdviceAttribute"; } }

      public IExtensionAttribute Create(Phx.Syms.AttrSym attrSym, Type attrType, AttributeHelper helper)
      {
         AdviceAttributeHandler newHandler = new AdviceAttributeHandler();
         AdviceAttribute attr = helper.InstantiateAttribute(attrSym, attrType) as AdviceAttribute;
         if (attr == null)
            return null;

         newHandler.advice = new PhxAdvice(attr);
         if (newHandler.advice.Pointcut == null)
         {
            Helpers.WriteError("Invalid pointcut '{0}'.", attr.Pointcut);
         }

         return newHandler;
      }

      /// <summary>
      /// The pointcut associated with this advice
      /// </summary>

      public PhxAdvice Advice
      {
         get { return this.advice; }
         set { this.advice = value; }
      }

      /// <summary>
      /// Called after attribute creation to specify the
      /// 'thing' (ownerSym) that this attribute is attached
      /// to and to initialize the attribute
      /// </summary>
      /// <param name="ownerSym">
      /// The Phx symbol that this attribute is attached to
      /// (cannot be null)</param>
      /// <param name="globalTypeTable">
      /// Type table used to resolve the type
      /// associated with the attribute
      /// </param>
      /// <returns>true on success</returns>

      public override bool
      Init
      (
         Phx.Syms.Sym ownerSym,
         Phx.Types.Table globalTypeTable
      )
      {
         // Execute the base class initialization

         if (!base.Init(ownerSym, globalTypeTable))
         {
            return false;
         }

         if (this.advice == null || this.advice.Pointcut == null)
         {
            return false;
         }

         // Hookup the pointcut method to call method this
         // attribute is attached to

         this.advice.AdviceMethodWeaver = 
            MethodEditorFactory.CreateEditor(ownerSym.AsFuncSym);

         return this.advice.AdviceMethodWeaver != null;
      }
   }
}
