using System.IO;
using System.Xml.Serialization;

using Phx.Aop;
using Phx.Aop.Attributes;

public class PersistenceAspect
{
   [Advice(AdviceType.after, "execution(* Person.new(..))")]
   public static void LogAfterCreation([This] Person This)
   {
      SaveChanges(This);
   }

   [Advice(AdviceType.after, "set(* * Person.*(..)) && !withincode(* Person.new(..))")]
   public static void SaveChanges([This] Person This)
   {
       FileStream fs = null;
       try
       {
          fs = new FileStream(This.Name + ".xml", FileMode.Create); 
          XmlSerializer xs = new XmlSerializer(This.GetType());
          xs.Serialize(fs, This);
       }
       catch (System.Exception)
       {
          //System.Console.WriteLine(ex);
       }
       finally
       {
          if (fs != null)
             fs.Dispose();
       }

       //EchoContents(This.Name);
   }

   public static void EchoContents(string name)
   {
       System.Console.WriteLine("----------------------------------------");
       System.Console.WriteLine("The contents of " + name + ".xml are:");
       try
       {
          TextReader tr = new StreamReader(name + ".xml");
          System.Console.WriteLine(tr.ReadToEnd());
          tr.Close();
       }
       catch
       { }

       System.Console.WriteLine("----------------------------------------");
       System.Console.WriteLine();
   }

   public static Person CreatePerson(string name)
   {
      Person p = null;

      FileStream fs = null;

      try
      {
         fs = new FileStream(name + ".xml", FileMode.Open, FileAccess.Read);
         XmlSerializer serializer = new XmlSerializer(typeof(Person));
         p = (Person) serializer.Deserialize(fs);
      }
      catch (System.Exception)
      {
         p = new Person(name);
      }
      finally
      {
         if (fs != null)
            fs.Dispose();
      }

      SaveChanges(p);

      return p;
   }
}

public class LogAspect
{
   [Advice(AdviceType.after, "execution(* Person.new(..))")]
   public static void LogAfterCreation([SourceLocationAttribute.WithinSignature] string withinSignature,
                     [SourceLocationAttribute.FileName] string fileName,
                     [SourceLocationAttribute.Line] uint line,
                     [This] Person This)
   {
      System.Console.WriteLine("--> Created person '{0}' [{1}(), File: {2}, Line: {3}]",
         This,
         withinSignature,
         fileName,
         line);
   }

   [Advice(AdviceType.before, "set(* * Person.*(..)) && !withincode(* Person.new(..))")]
   public static void LogBeforeChange([SourceLocationAttribute.WithinSignature] string withinSignature,
                     [SourceLocationAttribute.FileName] string fileName,
                     [SourceLocationAttribute.Line] uint line,
                     [This] Person This)
   {
      System.Console.WriteLine("--> Updating '{0}' [{1}(), File: {2}, Line: {3}]",
         This.Name,
         withinSignature,
         fileName,
         line);
      System.Console.WriteLine("    Before: " + This);
   }

   [Advice(AdviceType.after, "set(* * Person.*(..)) && !withincode(* Person.new(..))")]
   public static void LogAfterChange([SourceLocationAttribute.WithinSignature] string withinSignature,
                     [SourceLocationAttribute.FileName] string fileName,
                     [SourceLocationAttribute.Line] uint line,
                     [This] object This)
   {
      System.Console.WriteLine("    After : " + This);
   }
}

