@REM Use 'set _echo=on' to echo all the commands
@if not "%_ECHO%"=="on" @if not "%_ECHO%"=="ON" @echo off

setlocal

REM *****
REM ***** Verify that Phx environment has been setup
REM ***** 

if "%RDKROOT%"=="" echo Must setup Phx environment first && exit /B 1

REM *****
REM ***** Local vars
REM ***** 

set MORPH_DIR=%~dp0..
set WICCA_DIR=%~dp0..\..

if '%TARGET_DIR%'=='' set TARGET_DIR=%WICCA_DIR%\bin

REM *****
REM ***** Parse command-line options
REM ***** 

call "%MORPH_DIR%\scripts\checkswitch.bat" source %*
if not "%errorlevel%"=="0" set WEAVE_TYPE=source

call "%MORPH_DIR%\scripts\checkswitch.bat" breakpoint %*
if not "%errorlevel%"=="0" set WEAVE_TYPE=breakpoint

call "%MORPH_DIR%\scripts\checkswitch.bat" binary %*
if not "%errorlevel%"=="0" set WEAVE_TYPE=binary

call "%MORPH_DIR%\scripts\checkswitch.bat" sbinary %*
if not "%errorlevel%"=="0" set WEAVE_TYPE=sbinary

call "%MORPH_DIR%\scripts\checkswitch.bat" demo %*
set DEMO=%errorlevel%

REM The INDENT variable looks empty but actually contains spaces
if "%DEMO%"=="1" (
	set INDENT=      
) else (
	set INDENT=
)

call "%MORPH_DIR%\scripts\checkswitch.bat" quiet %*
set QUIET=%errorlevel%

call "%MORPH_DIR%\scripts\checkswitch.bat" skip_build_wicca %*
set SKIP_BUILD_WICCA=%errorlevel%

REM *****
REM ***** Verify required environment vars
REM ***** 

call "%MORPH_DIR%\scripts\requireenv.bat" TARGET %TARGET%
if not "%errorlevel%"=="0" exit /B 1

call "%MORPH_DIR%\scripts\requireenv.bat" PROJECT_DIR "%PROJECT_DIR%"
if not "%errorlevel%"=="0" exit /B 1

REM *****
REM ***** Execute test
REM *****

if "%DEMO%"=="1" (
	echo.
	echo ===== Build Wicca =====
	echo.
	set /p ANS=      Press enter to continue...
) else (
	echo Building - Wicca
)

REM I hate DOS!
call :clearerror

set MSBUILD_FLAGS=/nologo /property:WarningLevel=0 /verbosity:quiet /target:Build /property:Configuration=Debug

if "%SKIP_BUILD_WICCA%"=="0" (

	if "%DEMO%"=="1" (
		msbuild %MSBUILD_FLAGS% /noconsolelogger %WICCA_DIR%\Wicca.sln
	) else (
		msbuild %MSBUILD_FLAGS% %WICCA_DIR%\Wicca.sln
	)
) 

if not "%errorlevel%"=="0" goto :EOF

if not "%SKIP_BUILD_WICCA%"=="0" (
	if not "%QUIET%"=="1" echo Skipping building Wicca
)

if "%WEAVE_TYPE%"=="binary"     call :do_dynamic_weave %WEAVE_TYPE% %*
if "%WEAVE_TYPE%"=="breakpoint" call :do_dynamic_weave %WEAVE_TYPE% %*
if "%WEAVE_TYPE%"=="sbinary"    call :do_static_weave binary %*
REM if "%WEAVE_TYPE%"=="source" call :do_static_weave source %*

if "%WEAVE_TYPE%"=="" (

	REM Test: Dynamic binary weaving
	call :do_dynamic_weave binary %*
	if not "%errorlevel%"=="0" exit /B 1

	REM Test: Dynamic breakpoint weaving
	call :do_dynamic_weave breakpoint -nobuild %*
	if not "%errorlevel%"=="0" exit /B 1

	REM Test: Static binary weaving
	call :do_static_weave binary -nobuild %*
	if not "%errorlevel%"=="0" exit /B 1
)

if not "%errorlevel%"=="0" exit /B 1

exit /B 0

:do_dynamic_weave

setlocal

for %%i in (%EXCLUDE_WEAVERS%) do if "%%i"=="%1" (
	echo Skipping dynamic %1 weaving.
	exit /B 0
)

set InputAssembly=IGNORED
set MDBG_FLAGS1=-verbosity 1 -nologo !load wicca
set MDBG_FLAGS2=!wrun %TARGET%.exe %CMD_LINE% %ASPECTS% !wgo !quit

set TARGET_BAK=%TARGET%
set TARGET=%TARGET_BAK%.%1
set TEST_EXE="%TARGET_DIR%\mdbg" %MDBG_FLAGS1% !wtype %1 %MDBG_FLAGS2%

if not "%2"=="-nobuild" (
	call %PROJECT_DIR%\buildall.bat %*
	if not "%errorlevel%"=="0" exit /B 1
)

if "%QUIET%"=="0" (
	if "%DEMO%"=="1" (
		echo %INDENT%Testing - Dynamic %1 weaving
	) else (
		echo Testing - Dynamic %1 weaving
	)
)

call "%MORPH_DIR%\scripts\dotest.bat" -nobuild -noildasm %*
if not "%errorlevel%"=="0" exit /B 1

set TARGET=%TARGET_BAK%

exit /B 0

:do_static_weave

for %%i in (%EXCLUDE_WEAVERS%) do if "%%i"=="s%1" (
	echo Skipping static %1 weaving.
	exit /B 0
)

set InputAssembly=IGNORED
set TEST_EXE=%TARGET%.exe %CMD_LINE%

if not "%2"=="-nobuild" (
	call %PROJECT_DIR%\buildall.bat %*
	if not "%errorlevel%"=="0" exit /B 1
)

if "%QUIET%"=="0" (
	if "%DEMO%"=="1" (
		echo %INDENT%Testing - Static %1 weaving
	) else (
		echo Testing - Static %1 weaving
	)
)

call %PROJECT_DIR%\morph.bat %*
if not "%errorlevel%"=="0" exit /B 1

call "%MORPH_DIR%\scripts\dotest.bat" -nobuild -noildasm %*
if not "%errorlevel%"=="0" exit /B 1

exit /B 0

:clearerror
exit /B 0
