@REM Use 'set _echo=on' to echo all the commands
@if not "%_ECHO%"=="on" @if not "%_ECHO%"=="ON" @echo off

REM
REM Morphs an assembly.
REM
REM Environment required:
REM
REM    InputAssembly - Path of the assembly to morph (ex: orig.dll)
REM    OutputAssembly - Path for the morphed assembly (ex: new.dll)
REM    Refs - [Optional] Paths to extension and/or reference assemblies
REM	   (ex: /r:ext.dll) (Must be in the format /r:<path>[,<path>]*)
REM
REM Options:
REM
REM     -quiet         Supresses output of non error messages
REM	-break         Break into the debugger
REM     -assertbreak   Break into the debugger when an assertion is hit
REM                    (default is to exit on assert)
REM	-ignoreasserts Should we ignore asserts during morphing (not
REM                    recommended!)
REM	-noverify      Skips the peverify step
REM	-noildasm      Does not create ildasm files
REM
REM Returns:
REM
REM     0 if morph succeeds, 1 otherwise
REM
REM Ex:
REM
REM	morph.bat -quiet
REM

setlocal

REM *****
REM ***** Verify that Phx environment has been setup
REM ***** 

if "%RDKROOT%"=="" echo Must setup Phx environment first && exit /B 1

REM *****
REM ***** Local vars
REM ***** 

REM *****
REM ***** Sets BUILD, BUILD_EXT, PHX_REF, MSIL_REF, X86_REF, MSIL_RT_REF, X86_RT_REF
REM ***** 

call "%MORPH_DIR%\scripts\SetBuildAndPhxRefs.bat"

set MORPH_DIR=%~dp0..
set MORPH_BIN=%MORPH_DIR%\bin

REM *****
REM ***** Parse command-line options
REM ***** 

call "%MORPH_DIR%\scripts\checkswitch.bat" echo %*
if "%errorlevel%"=="1" (
 	set _ECHO=on
 	echo on
)

call "%MORPH_DIR%\scripts\checkswitch.bat" quiet %*
set QUIET=%errorlevel%

call "%MORPH_DIR%\scripts\checkswitch.bat" noildasm %*
set NO_ILDASM=%errorlevel%

call "%MORPH_DIR%\scripts\checkswitch.bat" assertbreak %*
if "%errorlevel%"=="1" (
	set ASSERT_BEHAVIOR=/assertbreak /Bd
) ELSE (
	set ASSERT_BEHAVIOR=/assertexit
)

call "%MORPH_DIR%\scripts\checkswitch.bat" ignoreasserts %*
if "%errorlevel%"=="1" (
	if "%ASSERT_BEHAVIOR%"=="/assertbreak" (
		echo Cannot specify -assertbreak and -ignoreasserts	
		exit /b 1
	) else (
		set ASSERT_BEHAVIOR=/assertlimit=0
	)
)

call "%MORPH_DIR%\scripts\checkswitch.bat" break %*
if "%errorlevel%"=="1" (
	set BREAK=/break:pereader
	set ASSERT_BEHAVIOR=/assertbreak /Bd
)

call "%MORPH_DIR%\scripts\checkswitch.bat" debug %*
if "%errorlevel%"=="1" (
	set DIFF=vf
	set ASSERT_BEHAVIOR=/assertbreak /Bd
)

call "%MORPH_DIR%\scripts\checkswitch.bat" demo %*
set DEMO=%errorlevel%

REM The INDENT variable looks empty but actually contains spaces
if "%DEMO%"=="1" (
	set INDENT=      
	set ASSERT_BEHAVIOR=/assertlimit=0
) else (
	set INDENT=
)

call "%MORPH_DIR%\scripts\checkswitch.bat" noverify %*
set NO_VERIFY=%errorlevel%

REM pereader doesn't support the assert flag for release builds
if not "%BUILD%"=="debug" set ASSERT_BEHAVIOR=

REM *****
REM ***** Verify required environment vars
REM ***** 

call "%MORPH_DIR%\scripts\requireenv.bat" InputAssembly %InputAssembly%
if not "%errorlevel%"=="0" exit /B 1

if '%OutputAssembly%'=='' (
	call :createtmpfile %InputAssembly%
	set OVERWRITE=1
) else (
	set OVERWRITE=0
)

REM This is totally bogus but for some reason the set OutputAssembly
REM above didn't work!  I hate DOS!

if '%OutputAssembly%'=='' (
	set OutputAssembly="%RESULT%"
)

REM *****
REM ***** Local vars
REM ***** 

call :getfilenamewthoutext %InputAssembly%

if "%OVERWRITE%"=="1" (
	call :getfilenamewthoutext %InputAssembly%
) else (
	call :getfilenamewthoutext %OutputAssembly%
)

set BEFORE_OUT="%RESULT%.orig.md"
set AFTER_OUT="%RESULT%.new.md"
set DIFF_OUT="%RESULT%.diff"

if "%PLUGIN_TO_USE%"=="" (
	set PLUGIN_TO_USE="%TARGET_DIR%\MorphPlugin.dll"
)

REM *****
REM ***** Copy the REFS to target dir and initialize REFS_OPTION
REM ***** 

set FILES_TO_COPY=%PEREADER% %PHX_REF% %MSIL_REF% %MSIL_RT_REF% %X86_REF% %X86_RT_REF%
call "%MORPH_DIR%\scripts\CopyAndAssignRefs.bat" %* %FILES_TO_COPY%

REM Determine PDB path

call :changeext %OutputAssembly% pdb
set PDB="%RESULT%"

call :changeext %InputAssembly% pdb
set PDB_NEW="%RESULT%"

if "%DEMO%"=="1" (
	echo.
	echo ===== Weave the assembly =====
	echo.
	set /p ANS=%INDENT%Press enter to see the original assembly...
	start ildasm %InputAssembly%
	echo.
	set /p ANS=%INDENT%Press enter to weave the assembly...
	echo.
)

if exist %PDB_NEW% (
	set PDB_FLAG=/pdb:%PDB%
) else (
	set PDB_FLAG=
)

call :displaymsg %InputAssembly% %OutputAssembly%

if not exist %PEREADER% (
	echo Must build pereader.exe first
	exit /B 1
)

REM Create the output directory if needed

call :getdir %OutputAssembly%
if not exist "%RESULT%" mkdir "%RESULT%"

REM *****
REM ***** Dump metadata for the original assembly
REM ***** 

if "%NO_ILDASM%"=="0" call :dumpassembly %InputAssembly% %BEFORE_OUT%

if "%OVERWRITE%"=="1" (

	call :createbackup %InputAssembly% orig
	call :createbackup %PDB_NEW% orig
)

REM *****
REM ***** MORPH THE ASSEMBLY
REM *****

pushd %MORPH_BIN%
if "%KEYFILE%" == "" set KEYFILE=%RDKROOT%rdk\setup\build\testkey.snk

if "%DEMO%"=="1" (
	echo.
	echo pereader.exe %ASSERT_BEHAVIOR% %BREAK% /plugin:%PLUGIN_TO_USE% %InputAssembly% /out:%OutputAssembly% %PDB_FLAG% %Refs%
    if exist "%KEYFILE%" echo sn.exe -Raq %OutputAssembly% "%KEYFILE%"
)

%PEREADER% %ASSERT_BEHAVIOR% %BREAK% /plugin:%PLUGIN_TO_USE% %InputAssembly% /out:%OutputAssembly% %PDB_FLAG% %Refs%
if exist "%KEYFILE%" sn.exe -Raq %OutputAssembly% "%KEYFILE%"
popd
if not "%errorlevel%"=="0" goto :EOF

rem ISSUE-TODO-WEIPINGH-2005/08/29-Phx#2680 investigate
rem For some reason pereader seems to not close the file properly.
rem We execute the ping command to busy wait until the file is closed.

ping localhost -n 2 > nul

REM *****
REM ***** Dump metadata for the new assebmly
REM ***** 

if "%NO_ILDASM%"=="0" call :dumpassembly %OutputAssembly% %AFTER_OUT%

if "%DEMO%"=="1" (
	echo.
	set /p ANS=%INDENT%Press enter to see the morphed assembly...
	start ildasm %OutputAssembly%
)

if "%OVERWRITE%"=="1" (

	copy %OutputAssembly% %InputAssembly% > nul
	if not '%PDB_FLAG%'=='' copy %PDB% %PDB_NEW% > nul

	REM Update the OutputAssembly for the remaining tests
	set OutputAssembly=%InputAssembly%
)

REM *****
REM ***** Verify morphed assembly
REM ***** 

if "%DEMO%"=="1" exit /B 0
if "%NO_VERIFY%"=="1" exit /B 0

peverify.exe /nologo /verbose %OutputAssembly% > peverify.out

for /f "delims= " %%i in (peverify.out) do if "%%i"=="All" goto :verified

if "%QUIET%"=="0" (
	for /f "delims=" %%i in (peverify.out) do echo %%i
)

del peverify.out

exit /B 1

:verified

del peverify.out

REM *****
REM ***** Diff the metadata
REM ***** 

if "%NO_ILDASM%"=="0" fc %BEFORE_OUT% %AFTER_OUT% > %DIFF_OUT%

exit /B 0

:dumpassembly
ildasm /text /metadata %1 > %2
call "%MORPH_DIR%\scripts\StripTokens.bat" %2
goto :eof

:displaymsg
if "%QUIET%"=="0" (
	if "%DEMO%"=="1" (
		echo %INDENT%Morphing %~xn1
	) else (
		echo Morphing - %1 to %2
	)
)
goto :eof

:parseans
echo =%1=
if "%1"=="" exit /b 0
if "%1"==" " exit /b 0
if "%1"=="n" exit /b 0
if "%1"=="N" exit /b 0
exit /b 1

:changeext
REM c:\joe.dll => c:\joe.exe
set RESULT=%~dpn1.%2
goto :eof

:getdir
REM c:\joe.dll => c:\
set RESULT=%~dp1
goto :eof

:createtmpfile
REM c:\joe.dll - c:\tmp\joe.dll
set RESULT=%~dp1tmp\%~nx1
goto :eof

:getfilenamewthoutext
set RESULT=%~dpn1
goto :eof

:createbackup
copy %~dpxn1 %~dpn1.orig%~x1 > nul
goto :eof
