//-----------------------------------------------------------------------------
//
// Copyright (C) Boriana Ditcheva.  All Rights Reserved.
//
// $Header: /Wicca/Wicca.wcs/WcsAdvice.cs 11    5/16/06 4:22p Bhd2105 $
//
// Description:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/Wicca.wcs/WcsAdvice.cs $
 * 
 * 11    5/16/06 4:22p Bhd2105
 * getAttributeIndexNeeded() returns -1 if no index has been specified for
 * the annotation. If -1 is returned, it is assumed that the first
 * annotation matching will be returned by default.
 * 
 * 10    5/14/06 4:35p Bhd2105
 * 
 * 9     5/11/06 11:32p Bhd2105
 * The Method_Decl_To_Call has been drastically changed. Now instead of
 * receiving a string argument that is passes along to the advice method
 * it's going to call, it receives an AttributeType as a string, and
 * the children arguments of the attribute in their ASTNode form. It
 * builds the method invocation and puts in as an arg a constructor of the
 * Attribute type with the args as args to the constructor. The invocation
 * it builds has this form:  className.method ( new
 * <attributeType>(<attrArg>, <attrArg>) );
 * 
 * 8     5/11/06 2:41p Bhd2105
 * 
 * 7     5/11/06 5:42a Bhd2105
 * Introduced arguments into the advice method
 * 
 * 6     5/09/06 1:10a Bhd2105
 * The check for method arguments was buggy. It checked for a child of
 * type ARG_LIST. Changed it to a check for child of type
 * FORMAL_PARAMETER_LIST, which is the correct one.
 * 
 * 5     5/08/06 4:18a Bhd2105
 * Implemented Method_Decl_To_Call method, which makes the method
 * declaration node stored in the class into a method call class for that
 * method.
 * 
 * 4     5/04/06 2:40p Bhd2105
 * 
 * 3     5/03/06 1:45a Bhd2105
 * Have added the node that will contain the advice method.
 * 
 * 2     4/24/06 6:36p Bhd2105
 */

using System;
using System.Collections.Generic;
using Phx.Aop.Attributes;


namespace Wcs.Aop
{
    public class WcsAdvice : Phx.Aop.Advice
    {

        public Kunle.CSharpParser.ASTNode adviceMethodNode = null;
        string className;


        public WcsAdvice(AdviceAttribute attr, Kunle.CSharpParser.ASTNode node, string cName)
            : base(attr)
        {
            adviceMethodNode = node;
            className = cName;
        }


        // Transform the method declaration node you have to a method call node
        // so you can insert the method call into the place where the joinpoint
        // matches the advice you have. It puts as arguments to the method call
        // the args passed to it.
        public Kunle.CSharpParser.ASTNode Method_Decl_To_Call(string type, List<Kunle.CSharpParser.ASTNode> args)
        {
            Kunle.CSharpParser.ASTNode callNode = new Kunle.CSharpParser.ASTNode();
            callNode.setType(Kunle.CSharpParser.CSharpParser.EXPR_STMT);
            callNode.setText(";");

            Kunle.CSharpParser.ASTNode invocNode = new Kunle.CSharpParser.ASTNode();
            invocNode.setType(Kunle.CSharpParser.CSharpParser.INVOCATION_EXPR);
            invocNode.setText("(");
            callNode.addChild(invocNode);

            Kunle.CSharpParser.ASTNode memberAccessNode = new Kunle.CSharpParser.ASTNode();
            memberAccessNode.setType(Kunle.CSharpParser.CSharpParser.MEMBER_ACCESS_EXPR);
            memberAccessNode.setText(".");
            invocNode.addChild(memberAccessNode);

            Kunle.CSharpParser.ASTNode id1Node = new Kunle.CSharpParser.ASTNode();
            id1Node.setType(Kunle.CSharpParser.CSharpParser.IDENTIFIER);
            id1Node.setText(className);
            memberAccessNode.addChild(id1Node);

            Kunle.CSharpParser.ASTNode id2Node = new Kunle.CSharpParser.ASTNode();
            id2Node.setType(Kunle.CSharpParser.CSharpParser.IDENTIFIER);
            id2Node.setText(adviceMethodNode.GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.IDENTIFIER).getText());
            memberAccessNode.addChild(id2Node);

            // If there are arguments, populate the arg list
            if (adviceMethodNode.GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.FORMAL_PARAMETER_LIST) != null)
            {
                Kunle.CSharpParser.ASTNode argListNode = new Kunle.CSharpParser.ASTNode();
                argListNode.setType(Kunle.CSharpParser.CSharpParser.ARG_LIST);
                invocNode.addChild(argListNode);

                Kunle.CSharpParser.ASTNode exprNode = new Kunle.CSharpParser.ASTNode();
                exprNode.setType(Kunle.CSharpParser.CSharpParser.EXPR);
                argListNode.addChild(exprNode);

                // The following nodes represent a new() expression as an argument to the
                // advice method call. It will have the form:
                // class.adviceMethod(new <AttributeName>(<args>));
                Kunle.CSharpParser.ASTNode newNode = new Kunle.CSharpParser.ASTNode();
                newNode.setType(Kunle.CSharpParser.CSharpParser.OBJ_CREATE_EXPR);
                newNode.setText("new");
                exprNode.addChild(newNode);

                Kunle.CSharpParser.ASTNode typeNode = new Kunle.CSharpParser.ASTNode();
                typeNode.setType(Kunle.CSharpParser.CSharpParser.TYPE);
                newNode.addChild(typeNode);

                Kunle.CSharpParser.ASTNode typeBase = new Kunle.CSharpParser.ASTNode();
                typeBase.setType(Kunle.CSharpParser.CSharpParser.IDENTIFIER);
                typeBase.setText(type);

                Kunle.CSharpParser.ASTNode starsBase = new Kunle.CSharpParser.ASTNode();
                starsBase.setType(Kunle.CSharpParser.CSharpParser.STARS);
                starsBase.setText("STARS");
                Kunle.CSharpParser.ASTNode array = new Kunle.CSharpParser.ASTNode();
                array.setType(Kunle.CSharpParser.CSharpParser.ARRAY_RANKS);
                array.setText("ARRAY_RANKS");
                typeNode.addChild(typeBase);
                typeNode.addChild(starsBase);
                typeNode.addChild(array);

                if (args != null)
                {
                    Kunle.CSharpParser.ASTNode subArgListNode = new Kunle.CSharpParser.ASTNode();
                    subArgListNode.setType(Kunle.CSharpParser.CSharpParser.ARG_LIST);
                    newNode.addChild(subArgListNode);


                    // Add the args to the method call
                    for (int i = 0; i < args.Count; i++)
                    {
                        subArgListNode.addChild(args[i]);
                    }
                }
            }   // End if-method-has-args


            return (callNode);
        }   // End Method_Decl_To_Call method


        // Return the index of the attribute the method will act on
        public int getAttributeIndexNeeded()
        {
            int index = -1;
            if (adviceMethodNode.GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.FORMAL_PARAMETER_LIST) != null &&
                adviceMethodNode.GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.FORMAL_PARAMETER_LIST).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.PARAMETER_FIXED).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.ATTRIBUTE_SECTIONS) != null &&
                adviceMethodNode.GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.FORMAL_PARAMETER_LIST).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.PARAMETER_FIXED).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.ATTRIBUTE_SECTIONS).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.ATTRIBUTE_SECTION).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.ATTRIBUTE).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.POSITIONAL_ARGLIST) != null)
            {
                index = int.Parse(adviceMethodNode.GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.FORMAL_PARAMETER_LIST).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.PARAMETER_FIXED).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.ATTRIBUTE_SECTIONS).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.ATTRIBUTE_SECTION).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.ATTRIBUTE).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.POSITIONAL_ARGLIST).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.POSITIONAL_ARG).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.ATTRIB_ARGUMENT_EXPR).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.EXPR).
                    GetFirstChildOfType(Kunle.CSharpParser.CSharpParser.INT_LITERAL).getText());
            }

            return (index);
        }

    }
}
