using System;
using System.Collections.Generic;

using Phx.Aop;
using Phx.Aop.Attributes;

public class TraceAspect
{
   static ConsoleColor colorToUse = ConsoleColor.Green;
   static Dictionary<int, Stack<ConsoleColor>> colorStackPerThread = 
      new Dictionary<int, Stack<ConsoleColor>>();

   [Advice(AdviceType.before, "call(* System.Console.WriteLine(..)) && within(*Aspect)")]
   public static void beforeWriteLine()
   {
      GetColorStackForCurrentThread().Push(System.Console.ForegroundColor);
      System.Console.ForegroundColor = colorToUse;
   }

   [Advice(AdviceType.after, "call(* System.Console.WriteLine(..)) && within(*Aspect)")]
   public static void afterWriteLine()
   {
      System.Console.ForegroundColor = GetColorStackForCurrentThread().Pop();
   }

   static Stack<ConsoleColor> GetColorStackForCurrentThread()
   {
      int threadId = System.Threading.Thread.CurrentThread.ManagedThreadId;

      Stack<ConsoleColor> stack;
      if (!colorStackPerThread.TryGetValue(threadId, out stack))
      {
         stack = new Stack<ConsoleColor>();
         colorStackPerThread.Add(threadId, stack);
      }

      return stack;
   }
}
