using Phx.Aop;
using Phx.Aop.Attributes;

public class LogAspect
{
   static System.ConsoleColor colorToUse = System.ConsoleColor.Green;

   [Advice(AdviceType.before, "call([Trace] * *.*(..))")]
   public static void logMethodCall(   [Signature] string signature,
                                       [Annotation(0)] object annotation0,
                                       [SourceLocationAttribute.FileName] string fileName,
                                       [SourceLocationAttribute.Line] uint line)
   {
      System.ConsoleColor fc = System.Console.ForegroundColor;
      System.Console.ForegroundColor = colorToUse;

      try
      {
         TraceAttribute traceAttr = annotation0 as TraceAttribute;

         if (traceAttr != null && traceAttr.msg != null && traceAttr.msg != string.Empty)
         {
            System.Console.WriteLine("{0}() called with message {1}. [{2},{3}]",
               signature, traceAttr.msg, fileName, line);
         }
         else
         {
            System.Console.WriteLine("{0}() called. [{1},{2}]",
               signature, fileName, line);
         }
      }
      finally
      {
         System.Console.ForegroundColor = fc;
      }
   }

   [Advice(AdviceType.before, "call([Note] * *.*(..))")]
   public static void logNote(   [Annotation(0)] object annotation0,
                                 [SourceLocationAttribute.FilePath] string fileName,
                                 [SourceLocationAttribute.Line] uint line)
   {
      System.ConsoleColor fc = System.Console.ForegroundColor;
      System.Console.ForegroundColor = colorToUse;

      try
      {
         System.Console.WriteLine("Note: {0}. [{1},{2}]",
            annotation0, fileName, line);
      }
      finally
      {
         System.Console.ForegroundColor = fc;
      }
   }

   [Advice(AdviceType.before, "call([Log] * *.*(..))")]
   public static void log( [Annotation(1)] LogAttribute annotation1,
                           [SourceLocationAttribute.FileName] string fileName,
                           [SourceLocationAttribute.Line] uint line)
   {
      System.ConsoleColor fc = System.Console.ForegroundColor;
      System.Console.ForegroundColor = colorToUse;

      try
      {
         System.Console.WriteLine("{0}: {1}. [{2},{3}]",
            annotation1.sev, annotation1.msg, fileName, line);
      }
      finally
      {
         System.Console.ForegroundColor = fc;
      }
   }
}
