using Phx.Aop;
using Phx.Aop.Attributes;

public class DisplayUpdatingAspect
{
	public static bool inited = false;

	[Advice(AdviceType.after, "call(void Shape.moveBy(int,int))")]
	[Advice(AdviceType.after, "call(void Point.moveBy(int,int))")]
	[Advice(AdviceType.after, "call(void Line.moveBy(int,int))")]
	public static void updateAfterMove( [SourceLocationAttribute.WithinSignature] string withinSignature,
                                            [SourceLocationAttribute.FileName] string fileName,
                                            [SourceLocationAttribute.Line] uint line)
	{
		//System.Console.WriteLine("updateAfterMove() called from {0}(). [{1},{2}]", 
		//	withinSignature, fileName, line);

		if (inited)
			Display.instance().update();
	}

	[Advice(AdviceType.after, "execution(public * SimpleDraw.new())")]
	public static void initDisplay( [This] object This,
                                        [SourceLocationAttribute.WithinSignature] string withinSignature,
                                        [SourceLocationAttribute.FileName] string fileName,
                                        [SourceLocationAttribute.Line] uint line)
	{
		//System.Console.WriteLine("initDisplay() called from {0}(). [{1},{2}]", 
		//	withinSignature, fileName, line);

		ISimpleDraw simpleDraw = This as ISimpleDraw;

		for (int i = 0; i < simpleDraw.Shapes.Length; i++)
			Display.instance().addShape(simpleDraw.Shapes[i]);

		Display.instance().update();

		inited = true;
	}
}
