//-----------------------------------------------------------------------------
//
// Phoenix
// Copyright (C) Microsoft Corporation.  All Rights Reserved.
//
// $Header: /Wicca/wicca/MdbgPlugin.cs 12    5/26/06 2:16p Eaddy $
//
// Description:
//
// Remarks:
//
//-----------------------------------------------------------------------------
/**
 * $Log: /Wicca/wicca/MdbgPlugin.cs $
 * 
 * 12    5/26/06 2:16p Eaddy
 * o Removed coordinator factory since we no
 * longer need the event mgr
 * 
 * 11    5/25/06 8:23p Eaddy
 * o Fixed FileTooLongEx that occurs when attempting
 * to get the directory name from a very long command
 * line string
 * 
 * 10    5/24/06 10:18p Eaddy
 * o Verbosity of 0 now surpresses all messages
 * 
 * 9     5/23/06 11:35p Eaddy
 * o Can now specify short names for weaver type
 * 
 * 8     5/23/06 11:30p Eaddy
 * o Now runs the process to completion so you don't
 * have to keep typing wgo
 * 
 * 7     5/23/06 7:49p Eaddy
 * o More cleanup
 * 
 * 6     5/11/06 9:49p Eaddy
 * o Now displays advice type
 * o Now forces output for some commands
 * 
 * 5     5/05/06 6:18p Eaddy
 * o Did some renaming to improve clarity
 * o Can now list current advice and enable/disable (although
 * enable/disable doesn't actually work yet)
 * 
 * 4     5/05/06 10:41a Eaddy
 * o Added remap and gas commands from the enc sample
 * 
 * 3     5/03/06 12:34a Eaddy
 * o Moved WriteOutput and WriteError into here
 * o weave command now requires at least one arg
 * 
 * 2     5/01/06 4:20p Eaddy
 * o Now assumes the working directory is the same as the client exe's
 * directory
 * instead of the current directory
 * 
 * 1     4/20/06 10:48p Eaddy
 * o Moved files from Wicca.Engine to wicca project
 * 
 * 26    4/20/06 6:36p Eaddy
 * o Wasn't retrieving the Verbosity correctly
 * 
 * 25    4/20/06 5:41p Eaddy
 * o Now gets the verbosity setting from mdbg.  However, to avoid
 * requiring an interface
 * change to mdbg, Wicca will use a default verbosity if the field isn't
 * present (we use
 * Reflection to look for it).  When this is made an official change to
 * mdbg we will stop
 * using Reflection.
 * 
 * 24    4/20/06 4:45p Eaddy
 * o Minor change
 * 
 * 23    4/19/06 7:13p Eaddy
 * 
 * 22    4/19/06 7:10p Eaddy
 * 
 * 21    4/19/06 1:40a Eaddy
 * o Made verbosity level more fine grained
 * o Now all output is done through a common method
 * 
 * 20    4/19/06 1:16a Eaddy
 * 
 * 19    4/18/06 11:41p Eaddy
 * o Added diagnostics message
 * o No longer tries to patch every time an assembly is loaded.
 * Must explicitly use the "patch" or "weave" commands.
 * o Fixed bug where a "patch" was ignored because the module
 * hadn't been loaded yet
 * 
 * 18    4/12/06 8:15p Eaddy
 * o Removed erroneous message
 * 
 * 17    4/12/06 7:37p Eaddy
 * o Removed event mgr stuff that was slowing down startup time
 * 
 * 16    4/12/06 4:03p Eaddy
 * o Fixed bug where process wasn't patched after applying aspect
 * 
 * 15    4/12/06 3:40p Eaddy
 * o Fixed identation
 * 
 * 14    4/11/06 3:39p Asv2105
 * Added ability to call code in debugged process
 * 
 * 13    4/03/06 12:52a Asv2105
 * Added a bunch of code for executing code in the client process when a
 * breakpoint is hit that doesn't entirely work
 * 
 * 12    4/02/06 4:34p Asv2105
 * Added ability for BreakpointWeaver to pass MethodInfo to the
 * coordinator
 * 
 * Adjusted ICoordinator, IEventListener interfaces to match reality
 * 
 * Added ability for Coordinator to call static methods when a breakpoint
 * fires and pass off to EventMgr for non-static methods
 * 
 * 11    3/31/06 5:24p Asv2105
 * Added initial EventMgr setup stuff
 * 
 * 10    3/31/06 4:43p Eaddy
 * o Added patch command
 * 
 * 9     3/31/06 1:36p Eaddy
 * o Changed bp to breakpoint
 * 
 * 8     3/15/06 4:56p Eaddy
 * o Removed my change since it caused normal case to not work.
 * Instead added wrun command.
 * 
 * 7     3/15/06 3:05p Eaddy
 * o Added prelim support for specifying weaver type from the command-line
 * 
 * 6     3/15/06 2:30p Eaddy
 * o Changed messages
 * 
 * 5     3/15/06 12:07p Eaddy
 * o Added "aspect" command for loading aspects interactively
 * o Added "verbose" command
 * 
 * 4     3/14/06 5:46p Eaddy
 * 
 * 3     3/03/06 10:34a Eaddy
 * o Integrated Adam's classes
 * 
 * 2     2/28/06 1:09a Eaddy
 * o Initial checkin
 */

using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;

using Dbg = System.Diagnostics.Debug;

using Microsoft.Samples.Tools.Mdbg;
using Microsoft.Samples.Debugging.MdbgEngine;

namespace Wicca
{
    [
       MDbgExtensionEntryPointClassAttribute(
          Url = "http://www.columbia.edu/~me133",
          EMailAddress = "me133@columbia.edu",
          ShortDescription = "Wicca Plugin for the Microsoft Managed Debugger sample (mdbg)"
       )
    ]
    public class WiccaPlugin : CommandBase
    {
        static Coordinator coordinator = new Coordinator();
        static int verbosity = 0;
        private static string g_lastSavedRunCommand = null;

        static int Verbosity
        {
            get { return verbosity; }
        }

        public static void LoadExtension()
        {
            if (Verbosity == 2)
            {
                WriteOutput(MDbgOutputConstants.StdOutput, "Wicca plugin loaded");
            }

            MDbgAttributeDefinedCommand.AddCommandsFromType(Shell.Commands, typeof(WiccaPlugin));

            PropertyInfo verbosityProperty = Shell.GetType().GetProperty("Verbosity");
            if (verbosityProperty != null)
                verbosity = (int)verbosityProperty.GetValue(WiccaPlugin.Shell, null);
            else
                verbosity = 0;

            coordinator.Init(verbosity);
        }

        static public void WriteOutput(string format, params object[] args)
        {
            if (WiccaPlugin.Verbosity >= 3)
            {
                string msg = string.Format("Wicca: " + format, args);
                Shell.IO.WriteOutput(MDbgOutputConstants.StdOutput, msg);
            }
        }

        static public void WriteOutputForce(string format, params object[] args)
        {
            // Temporarily increase verbosity to force our message to be
            // displayed
            PropertyInfo verbosityProperty = Shell.GetType().GetProperty("Verbosity");
            if (verbosityProperty != null)
                verbosityProperty.SetValue(WiccaPlugin.Shell, 2, null);

            string msg = string.Format("Wicca: " + format, args);
            Shell.IO.WriteOutput(MDbgOutputConstants.StdOutput, msg);

            if (verbosityProperty != null)
               verbosityProperty.SetValue(WiccaPlugin.Shell, Verbosity, null);
         }

        static public void WriteError(string format, params object[] args)
        {
            if (WiccaPlugin.Verbosity >= 1)
            {
                string msg = string.Format("Wicca: " + format, args);
                Shell.IO.WriteOutput(MDbgOutputConstants.StdError, msg);
            }
        }

        [
           CommandDescription(
              CommandName = "wrun",
              ShortHelp = "Runs a program under Wicca",
              MinimumAbbrev = 2,
              LongHelp = @"
Usage: wrun [-ver version_string] [[path_to_exe] [args_to_exe]]
    Kills the current process (if there is one) and starts a new one. If no
    executable argument is passed, this command runs the program that was
    previously executed with the run command. If the executable argument is
    provided, the specified program is run using the optionally supplied args.
    If class load, module load, and thread start events are being ignored (as
    they are by default), then the program will stop on the first executable
    instruction of the main thread.
    The -ver flag allows to explicitly specify version string that used to
    load correct debugger implementation. When the string is not entered it
    is deduced from the debugged program.
See Also:
    kill
"
           )
        ]
        public static void WRunCmd(string arguments)
        {
            ArrayList bpLocations = null;

            if (arguments.Length == 0 && g_lastSavedRunCommand != null)
            {
                // we have requested to restart the process.
                // We should kill the existing one.
                if (Debugger.Processes.HaveActive)
                {
                    // save locations of current breakpoints
                    foreach (MDbgBreakpoint b in Debugger.Processes.Active.Breakpoints)
                    {
                        if (bpLocations == null)
                        {
                            bpLocations = new ArrayList();
                        }
                        bpLocations.Add(b.Location);
                    }
                    ExecuteCommand("kill");
                }
                arguments = g_lastSavedRunCommand;
            }

            DebugModeFlag debugMode;
            string programToRun;
            string deeVersion;
            PreParseRunArguments(arguments, out debugMode, out deeVersion, out programToRun);
            if (deeVersion == null)
            {
                WriteOutput("Could not detect debuggee version -- using latest debugger API.");
            }

            // Override debug mode - always use EnC so we can support
            // Edit-and-Continue, source weaving, and binary weaving
            debugMode = DebugModeFlag.Enc;

            // create correct process object
            MDbgProcess p = Debugger.Processes.CreateLocalProcess(deeVersion);

            p.DebugMode = debugMode;

            // This flag must be set to allow a plugin to catch the module
            // load events
            p.RawModeType = MDbgProcess.RawMode.AlwaysStop;

            string workingDirectory;

            int exe = programToRun.IndexOf(".exe");
            if (exe != -1)
            {
                workingDirectory = Path.GetDirectoryName(programToRun.Substring(0, exe+4));
            }
            else
            {
                workingDirectory = Path.GetDirectoryName(programToRun);
            }

            if (workingDirectory == string.Empty)
                workingDirectory = ".";

            p.CreateProcess(null, programToRun, workingDirectory);

            // recreate debugger breakpoints
            if (bpLocations != null)
            {
                foreach (object location in bpLocations)
                {
                    p.Breakpoints.CreateBreakpoint(location);
                }
            }
            p.Go().WaitOne();
            g_lastSavedRunCommand = arguments;

            Shell.DisplayCurrentLocation();
        }

        [
           CommandDescription(
              CommandName = "wgo",
              ShortHelp = "Runs the process to completion",
              LongHelp = @"
Usage: wgo
    Runs the process to completion.
"
           )
        ]
        public static void WGoCmd(string args)
        {
            coordinator.RunToCompletion();
        }

        [
           CommandDescription(
              CommandName = "weave",
              ShortHelp = "Weave an aspect into the base program",
              MinimumAbbrev = 2,
              LongHelp = @"
Usage: weave module
    Loads the aspects defined in the module and attempts to
    apply them to the currently running process.

Usage: weave <advice id>
    Renables a previously an advice that was previously disabled
    via 'unweave'.
"
           )
        ]
        public static void WeaveCmd(string args)
        {
            ArgParser ap = new ArgParser(args);

            if (ap.Count < 0 || ap.Count > 1)
            {
                WriteError("Wrong amount of arguments.");
                return;
            }
            else if (ap.Count == 0)
            {
                int adviceID = 0;
                IEnumerable advices = coordinator.Advices;
                foreach(Phx.Aop.Advice advice in advices)
                {
                    WriteOutputForce("-----------------------------------------");
                    WriteOutputForce("Advice ID : " + ++adviceID + (advice.Enabled ? "" : " (DISABLED)"));
                    WriteOutputForce("Type      : " + advice.AdviceType);
                    WriteOutputForce("Pointcut  : " + advice.Pointcut);
                    WriteOutputForce("Advice    : " + advice.AdviceMethod);
                    WriteOutputForce("Applied to join points:");
                    
                    int jpID = 0;
                    foreach(string jp in advice.AppliedJoinpoints)
                    {
                        WriteOutputForce("  " + adviceID + "." + ++jpID + ". " + jp);
                    }
                }
                
                if (adviceID > 0)
                    WriteOutputForce("-----------------------------------------");
             }
            else if (ap.Count == 1)
            {
                string aspectModule = ap.AsString(0);
                int adviceToEnable = 0;

                if (!File.Exists(aspectModule))
                {
                    adviceToEnable = ap.AsInt(0);
                    if (adviceToEnable <= 0)
                    {
                        WriteError("Aspect file not found: " + aspectModule + ".");
                    }
                    else
                    {
                        EnableDisableAdvice(adviceToEnable, true);
                    }

                    return;
                }

                coordinator.LoadAspect(aspectModule);

                if (Verbosity == 2)
                {
                    WriteOutput("Aspect loaded.");
                }
            }
            else
            {
                System.Diagnostics.Debug.Fail("Can't get here");
            }
        }

        [
           CommandDescription(
              CommandName = "unweave",
              ShortHelp = "UnWeave an aspect from the base program",
              MinimumAbbrev = 1,
              LongHelp = @"
Usage: unweave <advice id>
    Disables the advice specified by the advice id.
    To reenable, use 'weave <advice id>'.
"
           )
        ]
        public static void UnWeaveCmd(string args)
        {
            ArgParser ap = new ArgParser(args);

            if (ap.Count <= 0 || ap.Count > 1)
            {
                WriteError("Wrong amount of arguments.");
                return;
            }
            else if (ap.Count == 1)
            {
                EnableDisableAdvice(ap.AsInt(0), false);
            }
            else
            {
                System.Diagnostics.Debug.Fail("Can't get here");
            }
        }

        static void EnableDisableAdvice(int adviceIDToSet, bool enable)
        {
            int adviceID = 0;
            IEnumerable advices = coordinator.Advices;

            if (adviceIDToSet <= 0)
            {
                WriteError("Illegal advice id: " + adviceIDToSet + 
                    ". Id must be greater than zero.");
                return;
            }

            if (!advices.GetEnumerator().MoveNext())
            {
                WriteError("There is no advice to unweave.");
                return;
            }

            foreach(Phx.Aop.Advice advice in advices)
            {
                if (adviceID + 1 == adviceIDToSet)
                {
                    advice.Enabled = enable;
                    WriteOutputForce((enable ? "Enabled" : "Disabled") + " advice: " + advice);
                    return;
                }
                ++adviceID;
            }

            WriteError("Unknown advice id: " + adviceIDToSet + 
                ". Use the 'weave' command to list the advice ids.");
        }

        [
           CommandDescription(
              CommandName = "patch",
              ShortHelp = "Patch the currently executing process",
              MinimumAbbrev = 2,
              LongHelp = @"
Usage: patch <PathToNewExe>
"
           )
        ]
        public static void PatchCmd(string args)
        {
            ArgParser ap = new ArgParser(args);
            if (ap.Count == 1)
            {
                string newProcessPath = ap.AsString(0);

                if (newProcessPath == null)
                {
                    WriteError("Must specify process path.");
                    return;
                }
                else if (newProcessPath == string.Empty)
                {
                    WriteError("Process path cannot be empty.");
                    return;
                }
                else if (!File.Exists(newProcessPath))
                {
                    WriteError("File does not exist: '" + newProcessPath + "'.");
                    return;
                }

                coordinator.CreateAndApplyDeltas(newProcessPath);
            }
            else if (ap.Count < 0 || ap.Count > 1)
            {
                WriteError("Wrong amount of arguments.");
                return;
            }
        }

        [
           CommandDescription(
              CommandName = "wtype",
              ShortHelp = "Set the weaver type",
              MinimumAbbrev = 2,
              LongHelp = @"
Usage: weaver [breakpoint|binary|source]
    breakpoint (default) - Breakpoint weaver (fast enable and disable
        but slow invocation) (short name: bp)
    binary - Binary weaver (slow enable and disable but fast invocation)
        (short name: bin)
    source - Source weaver (C# only) (very slow enable/disable, fast
        invocation, source-level debugging) (short name: s)
"
           )
        ]
        public static void WTypeCmd(string args)
        {
            ArgParser ap = new ArgParser(args);
            if (ap.Count == 1)
            {
                string weaverType = ap.AsString(0);
                try
                {
                    coordinator.WeaverType = (WeaverType)System.Enum.Parse(
                       typeof(WeaverType), weaverType);
                }
                catch (Exception ex)
                {
                    WriteError("Invalid weaver type '" +
                       weaverType + "': " + ex.Message + ".");
                    return;
                }
            }
            else if (ap.Count < 0 || ap.Count > 1)
            {
                WriteError("Wrong amount of arguments.");
                return;
            }

            if (Verbosity == 2)
            {
                WriteOutput("Weaver: " + coordinator.WeaverType);
            }
        }

        [
         CommandDescription(
                            CommandName="remap",
                            MinimumAbbrev=2,
                            ShortHelp="remap [new il] - remaps old version of function to the new one."
                            )
        ]
        public static void RemapCmd(string args)
        {
            ArgParser ap = new ArgParser(args);
            if(!(Debugger.Processes.Active.StopReason is RemapOpportunityReachedStopReason))
            {
                WriteError("Not stopped at RemapOpportunity breakpoint.");
                return;
            }

            RemapOpportunityReachedStopReason sr = (RemapOpportunityReachedStopReason)Debugger.Processes.Active.StopReason;
            if(!ap.Exists(0))
            {
                // no arguments -- we just print content of last remap callback
                // we want to do remap.
                WriteOutputForce("Last FunctionRemapOpportunity Callback:");
                WriteOutputForce("-----------------------------------------");
                WriteOutputForce("AppDomain:     " + sr.AppDomain.Name);
                WriteOutputForce("NewFunction:   " + sr.NewFunction.Token);
                WriteOutputForce("OldFunction:   " + sr.OldFunction.Token);
                WriteOutputForce("OldILOffset:   " + sr.OldILOffset);
                WriteOutputForce("Thread Number: " + Debugger.Processes.Active.Threads.Lookup(sr.Thread).Number);
            }
            else
            {
                int newILOffset = ap.AsInt(0);
                sr.Thread.ActiveFrame.RemapFunction(newILOffset);
                Debugger.Processes.Active.Threads.RefreshStack();
                WriteOutput("Remap successful.");
            }
        }

        [
         CommandDescription(
                            CommandName="funcVersion",
                            ShortHelp="prints current function version",
                            LongHelp=@"
Usage: funcversion
    Prints current version of the function.
"
                            )
         ]
        public static void FuncVersionCmd(string args)
        {
            ArgParser ap = new ArgParser(args);
            MDbgFunction function;
            if(ap.AsString(0)==".")
                function = Debugger.Processes.Active.Threads.Active.CurrentFrame.Function;
            else
                function = Debugger.Processes.Active.ResolveFunctionNameFromScope(ap.AsString(0));
             
            WriteOutputForce("Current version: " + function.CorFunction.Version);
        }
    }
}
