///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Common/AssemblyHelper/MethodInfo.h 7     5/10/06 8:02p Eaddy $
//
//    File name:	MethodInfo.h
//   Created By:	Marc Eaddy
//   Created on:	10/24/04
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Common/AssemblyHelper/MethodInfo.h $
 * 
 * 7     5/10/06 8:02p Eaddy
 * o Closer to multiple weaving
 * 
 * 6     4/22/06 12:35p Eaddy
 * o Adding support for using the Debugger API to query for the current
 * metadata and IL
 * 
 * 5     4/18/06 11:43p Eaddy
 * o Changed back to the old emit algorithm
 * o Improved diagnostics
 * o No longer outputs empty dil or dmeta files
 * 
 * 4     4/14/06 5:36p Eaddy
 * o Trying to fix emitting methods
 * 
 * 3     4/11/06 11:12p Eaddy
 * o Dynamic binary updating proof-of-concept works!
 * 
 * 2     4/07/06 8:54p Eaddy
 * o Now supports outputing updated methods to a char buffer
 * in addition to using IMemoryAlloc
 * 
 * 1     3/16/06 1:53a Eaddy
 * 
 * 1     3/16/06 1:49a Eaddy
 * 
 * 10    1/20/06 9:39p Eaddy
 * o Migrated to VS 2005.  Replaced many function calls with their safe
 * version.
 * 
 * 9     12/16/05 8:41p Eaddy
 * o Added isCtor and isCCtor
 * 
 * 8     12/15/05 2:37a Eaddy
 * o Improved statistics gathering and display
 * 
 * 7     12/10/05 12:41a Eaddy
 * o Now each patch keeps track of its time
 * 
 * 6     12/09/05 9:03p Eaddy
 * o Made improvements to try to emit methods but this doesn't work
 * because DefineMethod must be called during ModuleLoadFinished
 * 
 * 5     12/09/05 4:32p Eaddy
 * o No longer depends on methodDef_iterator
 * 
 * 4     12/08/05 10:34p Eaddy
 * o Now really fixed problem with changing a method's local variables
 * 
 * 3     5/28/05 1:20p Eaddy
 * Changed copyright
 * Revision 1.2  2004/10/26 04:43:16  me133
 * no message
 *
 * Revision 1.1  2004/10/25 01:03:49  me133
 * o Method IL comparison is now complete and correct
 *
 */
#ifndef INCLUDED_METHOD_INFO_H
#define INCLUDED_METHOD_INFO_H

#include "corhlpr.h"			// IMetaDataImport, mdMethodDef

#include <cor.h>				// Needed to compile corprof.h!
#include <corprof.h>			// ICorProfilerInfo

#include <atlcomcli.h>			// CComPtr

#include <list>					// STL list<>

#include "ListTokenMappings.h"  // TypeListTokenMappings

// Forward declarations

class CILFetcher;

class CMethodInfo
{
public:

	CMethodInfo();
	CMethodInfo(const CMethodInfo& that);
   ~CMethodInfo();

	HRESULT init(ICorProfilerInfo* pProfInfo, FunctionID functionID, const WCHAR* szTypeName, const CILFetcher* pilFetcher);
	HRESULT init(IMetaDataImport* piImport, mdMethodDef methodDef, const WCHAR* szTypeName, const CILFetcher* pilFetcher);

	void clear();

    HRESULT cloneMethodInfo(const CMethodInfo& methodInfoToClone,
                            IMethodMalloc* piAlloc);

	LPBYTE cloneMethodHeader(	IMethodMalloc* piAlloc, 
								const TypeListTokenMappings& listEquivalentTokens,
                                bool* pbMustDelete) const;

    void cloneMethodHeader(	    IMethodMalloc* piAlloc,
                                LPBYTE* ppbMethodHeaderNew,
                                bool* pbMustDelete) const;

	HRESULT emit(	IMetaDataEmit* piMetaDataEmit, 
					mdToken tokEnclosingScope, 
					mdToken* ptokEmitted);

	HRESULT emit(	IMetaDataEmit* piMetaDataEmit, 
					mdToken* ptokEmitted);

	HRESULT update(		const CMethodInfo& methodNew,
						ModuleID moduleID,
						FunctionID functionID,
						IMethodMalloc* piAlloc,
						IMetaDataEmit* piEmit,
						const TypeListTokenMappings& listEquivalentTokens);

	HRESULT updateSignature(	mdSignature tokSignatureNew,
								mdSignature* ptokSignature,
								PCCOR_SIGNATURE* pcorSigBlob,
								ULONG* pulBlobLen,
								const TypeListTokenMappings& listEquivalentTokens);

	const WCHAR* toString() const;
	const WCHAR* toDump() const;
	
	friend bool operator<(const CMethodInfo& lhs, const CMethodInfo& rhs);

	bool isPInvoke() const;
	bool isAbstract() const;
	bool isCtor() const;
	bool isCCtor() const;

	LPBYTE getCodeStart() const;
	LPBYTE getCodeEnd() const;

	const WCHAR* getTypeName() const;

private: // HELPER FUNCTIONS
	CMethodInfo& operator=(const CMethodInfo& that);

public: // DATA

	static const int nMAX_ID = 2048; // from alink.h (Rotor)

	CComPtr<IMetaDataImport> m_spiImport;
    const CILFetcher* m_pilFetcher;

	mdTypeDef m_typeDef;
	mdMethodDef m_methodDef;
	mutable WCHAR m_szTypeName[nMAX_ID];
	WCHAR m_szMemberName[nMAX_ID];
	ULONG m_ulNameLen;
	DWORD m_dwFlags;

	mdSignature m_tokSig;
	PCCOR_SIGNATURE m_pbSigBlob;
	ULONG m_ulSigBlob;

	ULONG m_ulCodeRVA;
	ULONG m_ulImplFlags;

	unsigned __int8* m_pbMethodHeader;
	bool m_bMustDelete;
	ULONG m_ulMethodSize;
	ULONG m_ulMethodHeaderSize;
	LPCBYTE m_pbMethodIL;
	ULONG m_ulCodeSize;

	mdSignature m_tokLocalVarSig;
	PCCOR_SIGNATURE m_pLocalVarSig;
	ULONG m_ulLocalVarSigSize;

	// Needed for method patching
	ModuleID m_moduleID;
	FunctionID m_functionID;
	bool m_bWasModified;
	DWORD m_dwTicksStart;
	DWORD m_dwWeaveTimeEnd;
	DWORD m_dwCompileTimeEnd;
	DWORD m_dwPatchTimeEnd;
	DWORD m_dwWaitForReJitEnd;
};

HRESULT GetMethodNameFromFunctionId(	ICorProfilerInfo* pProfInfo,
										FunctionID functionId,
										wchar_t* wszClass, 
										wchar_t* wszMethod,
										DWORD dwSize);

// Returns S_OK if the methods are equivalent, S_FALSE if the methods are
// not equivalent, and E_FAIL if a comparison couldn't be made
HRESULT MethodsAreEquivalent(	const CMethodInfo& pre, 
								const CMethodInfo& post, 
								const TypeListTokenMappings& listNewOrReorderedStrings);

#endif // INCLUDED_METHOD_INFO_H
